import os
import sys
import numpy as np
from matplotlib import pyplot as plt

cross_x = []
cross_y = []
halo_y = []
is_fvm_map = int(sys.argv[1]) if len(sys.argv) > 1 else 0
fvm_vertices_init = []
fvm_ranks = []
fvm_vertices = []
fvm_elements = []
fvm_max_borders = []
n_data_exchanged = 1
n_fvm_vertices = 3
font_size = 15
plt.rcParams.update({'font.size': font_size})

if is_fvm_map:
	with open('../fvm-map/out/modified/benchmark.txt', 'r') as f:
		lines = [line.strip().split() for line in f.readlines()]
		for line in lines:
			fvm_vertices_init += [int(line[0].split('-')[-1][:-5])]
			fvm_ranks += [int(line[2])]
			fvm_vertices += [int(line[4])]
			fvm_elements += [int(line[5])]
			max_border = max(int(line[6]), int(line[7]))
			fvm_max_borders += [max_border]
			cross_x += [max_border * 8 * n_data_exchanged]
			cross_y += [-1]
			halo_y += [float(line[12])]
	if len(set(fvm_max_borders)) >= n_fvm_vertices:
		order = np.argsort(fvm_max_borders)
		fvm_max_border_order = list(np.array(fvm_max_borders)[order])
		indexes = [len(order)-1]
		max_values = [fvm_max_border_order[-1]]
		for i in range(len(fvm_max_border_order)-2, -1, -1):
			if len(indexes) == n_fvm_vertices:
				break
			if fvm_max_border_order[i] not in max_values:
				indexes += [i]
				max_values += [fvm_max_border_order[i]]
		indexes = [order[i] for i in indexes]
		fvm_vertices_init = [fvm_vertices_init[i] for i in indexes]
		fvm_ranks = [fvm_ranks[i] for i in indexes]
		fvm_vertices = [fvm_vertices[i] for i in indexes]
		fvm_elements = [fvm_elements[i] for i in indexes]
		fvm_max_borders = [fvm_max_borders[i] for i in indexes]
		cross_x = [cross_x[i] for i in indexes]
		cross_y = [cross_y[i] for i in indexes]
		halo_y = [halo_y[i] for i in indexes]
elif len(sys.argv) > 2:
	cross_x = [int(arg) * 8 * n_data_exchanged for arg in sys.argv[2].split()]
	cross_y = [-1] * len(cross_x)
n_arg = len(cross_x)

out_dir = "out/"
filedir_inputs = os.listdir(out_dir)
filedir_outputs = [filename[:-4] + ".png" for filename in filedir_inputs]
filedir_inputs = [out_dir + filename for filename in filedir_inputs]
filedir_outputs = ["plt/" + filename for filename in filedir_outputs]

collected_runtimes = {}
coll_labels_true = False

n_files = len(filedir_inputs)
for i in range(n_files):
	array_sizes = []
	runtimes = []#ms
	P=-1

	with open(filedir_inputs[i], 'r') as f:
		lines = f.readlines()
		lines = [line.split() for line in lines]
		P = int(lines[0][0])
		lines = lines[1:]
		array_sizes = [int(line[0]) for line in lines]
		runtimes = [float(line[1]) for line in lines]
		coll_file = filedir_inputs[i][len(out_dir):-4]
		coll_file_split = coll_file.split('-')
		coll_n = int(coll_file_split[0].split('n')[0])
		coll_p = int(coll_file_split[1].split('p')[0])
		coll_t = int(coll_file_split[2].split('t')[0])
		if len(coll_file_split) > 3:
			coll_labels_true = True
			coll_label = int(coll_file_split[3])
			coll_key = (coll_n, coll_p, coll_t, coll_label)
		else:
			coll_key = (coll_n, coll_p, coll_t)
		collected_runtimes[coll_key] = {"xs":array_sizes, "ys":runtimes}

	coef = np.polyfit(array_sizes, runtimes, 1)
	poly1d_fn = np.poly1d(coef)

	for j in range(n_arg):
		cross_y[j] = poly1d_fn(cross_x[j])
		print(filedir_outputs[i], (cross_x[j], cross_y[j]))
		if not is_fvm_map:
			plt.vlines(x=cross_x[j], ymin=0, ymax=cross_y[j], linestyle=':')
	plt.plot(array_sizes, runtimes, 'o', array_sizes, poly1d_fn(array_sizes), '--', color='k')
	for j in range(n_arg):
		if is_fvm_map:
			label = 'V:{},R:{},borders:{}'.format(fvm_vertices_init[j], fvm_ranks[j], fvm_max_borders[j])
			plt.plot([cross_x[j]], [halo_y[j]], 'o', label=label)
		else:
			plt.plot([cross_x[j]], [cross_y[j]], 'o')
	if is_fvm_map:
		plt.legend()
	plt.xlabel('Message size [B]')
	plt.ylabel('Communication time [ms]')
	plt.ylim(bottom=0)
	if os.path.isfile(filedir_outputs[i]):
		os.remove(filedir_outputs[i])
	plt.savefig(filedir_outputs[i], bbox_inches='tight')
	plt.clf()

coll_labels = ["Same CPU", "Different CPU", "Different nodes", "4 nodes", "8 nodes", "", ""]
coll_keys_sorted = {}
if coll_labels_true:
	coll_keys_sorted = sorted(collected_runtimes, key=lambda x: x[3], reverse=True)
else:
	coll_keys_sorted = sorted(collected_runtimes, key=lambda x: (x[1], x[0], x[2]))
for coll_key in coll_keys_sorted:
	value = collected_runtimes[coll_key]
	coll_label = "N:{}, R:{}".format(coll_key[0], coll_key[1])
	if len(coll_key) > 3:
		coll_label = coll_labels[coll_key[3]]
	plt.plot(value["xs"][1:], value["ys"][1:], 'o', label=coll_label)
plt.legend()
plt.xlabel('Message size [B]')
plt.ylabel('Communication time [ms]')
plt.savefig('plt/collected_mpi_test.png', bbox_inches='tight')
plt.clf()
