#include "compute_test.h"

#define BILLION 1e9
#define random(max_random) ((real_t) rand()/(real_t) (RAND_MAX)) * (max_random);
#define pow2(x) ((x)*(x))

int rank, size;

int_t
	n_tests = 50,
	n_pingpong_tests = 1e7,
	*array_sizes = NULL,
	n_sizes = 100,
	len_array = 2e3,
	max_random = 1,
	len_warm_up = 1e7;

real_t
	start_time,
	end_time,
	current_time,
	*time_comm = NULL,
	*data_array0 = NULL,
	*data_array1 = NULL,
	*data_array2 = NULL,
	*data_array3 = NULL,
	*data_array4 = NULL,
	*data_array5 = NULL,
	*data_array6 = NULL,
	*data_array7 = NULL,
	*data_array8 = NULL,
	*data_array9 = NULL,
	*data_array10 = NULL,
	*data_array11 = NULL,
	*data_arrays[12] = {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};


void init ( void )
{
	MPI_Init(NULL, NULL);
	MPI_Comm_size(MPI_COMM_WORLD, &size);
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);

	time_comm = malloc (n_tests * sizeof(real_t));

	data_array0 = malloc (len_array * sizeof(real_t));
	data_array1 = malloc (len_array * sizeof(real_t));
	data_array2 = malloc (len_array * sizeof(real_t));
	data_array3 = malloc (len_array * sizeof(real_t));
	data_array4 = malloc (len_array * sizeof(real_t));
	data_array5 = malloc (len_array * sizeof(real_t));
	data_array6 = malloc (len_array * sizeof(real_t));
	data_array7 = malloc (len_array * sizeof(real_t));
	data_array8 = malloc (len_array * sizeof(real_t));
	data_array9 = malloc (len_array * sizeof(real_t));
	data_array10 = malloc (len_array * sizeof(real_t));
	data_array11 = malloc (len_array * sizeof(real_t));
	data_arrays[0] = data_array0;
	data_arrays[1] = data_array1;
	data_arrays[2] = data_array2;
	data_arrays[3] = data_array3;
	data_arrays[4] = data_array4;
	data_arrays[5] = data_array5;
	data_arrays[6] = data_array6;
	data_arrays[7] = data_array7;
	data_arrays[8] = data_array8;
	data_arrays[9] = data_array9;
	data_arrays[10] = data_array10;
	data_arrays[11] = data_array11;

	array_sizes = malloc (n_sizes * sizeof(int_t));

	for (int_t i=0; i<12; i++)
		for (int_t j=0; j<len_array; j++)
			data_arrays[i][j] = random(max_random);

	int_t size = len_array / n_sizes;
	for (int_t i=0; i<n_sizes; i++) {
		array_sizes[i] = size;
		size += len_array / n_sizes;
	}
}


void finalize ( void )
{
	free(time_comm);

	free(data_array0);
	free(data_array1);
	free(data_array2);
	free(data_array3);
	free(data_array4);
	free(data_array5);
	free(data_array6);
	free(data_array7);
	free(data_array8);
	free(data_array9);
	free(data_array10);
	free(data_array11);
	free(array_sizes);

	MPI_Finalize();
}


void warm_up ( void )
{
	#pragma omp parallel for
	for (int_t i=0; i<len_warm_up; i++) {
		int_t index = i % len_array;
		data_array0[index] = data_array1[index] * data_array2[index] / (data_array3[index] * data_array4[index]);
		data_array5[index] = data_array6[index] / data_array7[index];
		data_array8[index] = data_array9[index] + data_array10[index] - data_array11[index];
	}
}


void pingpong_test ( void )
{
	real_t
		start,
		end,
		latency,
		latency_reduced,
		ping,
		pong;

	int_t peer = (rank + 1) % (size);
	ping = random(max_random);

	int_t n_reqs = 2;
	MPI_Request reqs[n_reqs];
	MPI_Status stats[n_reqs];

	MPI_Barrier(MPI_COMM_WORLD);
	start = MPI_Wtime();
	for (int_t i=0; i<n_pingpong_tests; i++) {
		MPI_Isend(&ping, 1, MPI_DOUBLE, peer, 0, MPI_COMM_WORLD, &reqs[0]);
		MPI_Irecv(&pong, 1, MPI_DOUBLE, peer, 0, MPI_COMM_WORLD, &reqs[1]);

		MPI_Waitall(n_reqs, reqs, stats);
		MPI_Barrier(MPI_COMM_WORLD);
	}
	end = MPI_Wtime();

	latency = (end - start) / n_pingpong_tests * 1e6;

	MPI_Barrier(MPI_COMM_WORLD);
	MPI_Reduce(&latency, &latency_reduced, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
	latency_reduced /= size;
	if (rank == 0) {
		FILE *f = fopen("latency.txt", "a");
		fprintf(f, "%lf [micro seconds]\n", latency_reduced);
		fclose(f);
	}
}


void communication_test ( void )
{
	int_t peer0 = (rank + 1) % (size);
	int_t peer1 = (rank - 1 + size) % (size);
	real_t avg_time = 0.0;
	real_t var_time = 0.0;

	for (int_t i=0; i<n_sizes; i++) {
		//if (rank==0) printf("Array of %ld\n", array_sizes[i]);
		for (int_t j=0; j<n_tests; j++) {
			warm_up();
			MPI_Barrier(MPI_COMM_WORLD);
			MPI_Request reqs[12];
			MPI_Status stats[12];
			int_t n_reqs = 0;
			MPI_Barrier(MPI_COMM_WORLD);

			start_time = MPI_Wtime();

			MPI_Issend(&data_array0[0], array_sizes[i], MPI_DOUBLE, peer0, rank+(size*0), MPI_COMM_WORLD, &reqs[n_reqs++]);
			MPI_Issend(&data_array1[0], array_sizes[i], MPI_DOUBLE, peer0, rank+(size*2), MPI_COMM_WORLD, &reqs[n_reqs++]);
			MPI_Issend(&data_array2[0], array_sizes[i], MPI_DOUBLE, peer0, rank+(size*4), MPI_COMM_WORLD, &reqs[n_reqs++]);
			MPI_Issend(&data_array3[0], array_sizes[i], MPI_DOUBLE, peer1, rank+(size*1), MPI_COMM_WORLD, &reqs[n_reqs++]);
			MPI_Issend(&data_array4[0], array_sizes[i], MPI_DOUBLE, peer1, rank+(size*3), MPI_COMM_WORLD, &reqs[n_reqs++]);
			MPI_Issend(&data_array5[0], array_sizes[i], MPI_DOUBLE, peer1, rank+(size*5), MPI_COMM_WORLD, &reqs[n_reqs++]);

			MPI_Irecv(&data_array6[0], array_sizes[i], MPI_DOUBLE, peer0, peer0+(size*1), MPI_COMM_WORLD, &reqs[n_reqs++]);
			MPI_Irecv(&data_array7[0], array_sizes[i], MPI_DOUBLE, peer0, peer0+(size*3), MPI_COMM_WORLD, &reqs[n_reqs++]);
			MPI_Irecv(&data_array8[0], array_sizes[i], MPI_DOUBLE, peer0, peer0+(size*5), MPI_COMM_WORLD, &reqs[n_reqs++]);
			MPI_Irecv(&data_array9[0], array_sizes[i], MPI_DOUBLE, peer1, peer1+(size*0), MPI_COMM_WORLD, &reqs[n_reqs++]);
			MPI_Irecv(&data_array10[0], array_sizes[i], MPI_DOUBLE, peer1, peer1+(size*2), MPI_COMM_WORLD, &reqs[n_reqs++]);
			MPI_Irecv(&data_array11[0], array_sizes[i], MPI_DOUBLE, peer1, peer1+(size*4), MPI_COMM_WORLD, &reqs[n_reqs++]);

			MPI_Waitall(n_reqs, reqs, stats);

			end_time = MPI_Wtime();

			current_time = end_time - start_time;
			MPI_Barrier(MPI_COMM_WORLD);
			//printf("RANK%d: Exchanging real_t array of %ld: %lf seconds\n", rank, array_sizes[i], current_time);
			MPI_Barrier(MPI_COMM_WORLD);
			MPI_Reduce(&current_time, &avg_time, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
			time_comm[j] = avg_time / size;
			//if (rank==0) printf("Average for round %ld: %lf\n", j, time_comm[j]);
		}
		avg_time = 0.0;
		var_time = 0.0;
		for (int_t j=0; j<n_tests; j++) {
			avg_time += time_comm[j];
		}
		avg_time /= n_tests;
		for (int_t j=0; j<n_tests; j++) {
			var_time += pow2(time_comm[i] - avg_time);
		}
		var_time /= n_tests!=1 ? ( n_tests - 1 ) : 1;
		var_time = sqrt ( var_time );
		if (rank==0) {
			printf("%ld %lf %lf\n", array_sizes[i]*8, avg_time*1000, var_time*1000);
		}
	}
}


void communication_test_block ( void )
{
	int_t peer0 = (rank + 1) % (size);
	int_t peer1 = (rank - 1 + size) % (size);
	real_t avg_time = 0.0;
	real_t var_time = 0.0;

	for (int_t i=0; i<n_sizes; i++) {
		//if (rank==0) printf("Array of %ld\n", array_sizes[i]);
		for (int_t j=0; j<n_tests; j++) {
			warm_up();
			MPI_Barrier(MPI_COMM_WORLD);

			start_time = MPI_Wtime();

			MPI_Sendrecv(&data_array0[0], array_sizes[i], MPI_DOUBLE, peer0, rank+(size*0),
						 &data_array9[0], array_sizes[i], MPI_DOUBLE, peer1, peer1+(size*0), MPI_COMM_WORLD, MPI_STATUS_IGNORE);
			MPI_Sendrecv(&data_array1[0], array_sizes[i], MPI_DOUBLE, peer0, rank+(size*2),
						 &data_array10[0], array_sizes[i], MPI_DOUBLE, peer1, peer1+(size*2), MPI_COMM_WORLD, MPI_STATUS_IGNORE);
			MPI_Sendrecv(&data_array2[0], array_sizes[i], MPI_DOUBLE, peer0, rank+(size*4),
						 &data_array11[0], array_sizes[i], MPI_DOUBLE, peer1, peer1+(size*4), MPI_COMM_WORLD, MPI_STATUS_IGNORE);
			MPI_Sendrecv(&data_array3[0], array_sizes[i], MPI_DOUBLE, peer1, rank+(size*1),
						 &data_array6[0], array_sizes[i], MPI_DOUBLE, peer0, peer0+(size*1), MPI_COMM_WORLD, MPI_STATUS_IGNORE);
			MPI_Sendrecv(&data_array4[0], array_sizes[i], MPI_DOUBLE, peer1, rank+(size*3),
						 &data_array7[0], array_sizes[i], MPI_DOUBLE, peer0, peer0+(size*3), MPI_COMM_WORLD, MPI_STATUS_IGNORE);
			MPI_Sendrecv(&data_array5[0], array_sizes[i], MPI_DOUBLE, peer1, rank+(size*5),
						 &data_array8[0], array_sizes[i], MPI_DOUBLE, peer0, peer0+(size*5), MPI_COMM_WORLD, MPI_STATUS_IGNORE);

			end_time = MPI_Wtime();

			current_time = end_time - start_time;
			MPI_Barrier(MPI_COMM_WORLD);
			//printf("RANK%d: Exchanging real_t array of %ld: %lf seconds\n", rank, array_sizes[i], current_time);
			MPI_Barrier(MPI_COMM_WORLD);
			MPI_Reduce(&current_time, &avg_time, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
			time_comm[j] = avg_time / size;
			//if (rank==0) printf("Average for round %ld: %lf\n", j, time_comm[j]);
		}
		avg_time = 0.0;
		var_time = 0.0;
		for (int_t j=0; j<n_tests; j++) {
			avg_time += time_comm[j];
		}
		avg_time /= n_tests;
		for (int_t j=0; j<n_tests; j++) {
			var_time += pow2(time_comm[i] - avg_time);
		}
		var_time /= n_tests!=1 ? ( n_tests - 1 ) : 1;
		var_time = sqrt ( var_time );
		if (rank==0) {
			printf("%ld %lf %lf\n", array_sizes[i]*8, avg_time, var_time);
		}
	}
}


void compute_test ( void )
{
	init();

	if (rank==0)
		printf("%d processes\n", size);
	MPI_Barrier(MPI_COMM_WORLD);

	warm_up();
	//pingpong_test();
	communication_test();
	//communication_test_block();

	finalize();
}

