import os
import sys
import numpy as np
from matplotlib import pyplot as plt


font_size=15
plt.rcParams.update({'font.size': font_size})

is_fvm_map = int(sys.argv[1]) if len(sys.argv) > 1 else 0
is_mkl = int(sys.argv[2]) if len(sys.argv) > 2 else 0

filedir_inputs = os.listdir("out/")
filedir_outputs_mem = {"out/" + filedir: "plt/" + filedir + "_mem.png" for filedir in filedir_inputs}
filedir_outputs_flops = {"out/" + filedir: "plt/" + filedir + "_flops.png" for filedir in filedir_inputs}
filedir_outputs_roof = {"out/" + filedir: "plt/" + filedir + "_roof.png" for filedir in filedir_inputs}
filedir_outputs_roof_zoom = {"out/" + filedir: "plt/" + filedir + "_roof_zoom.png" for filedir in filedir_inputs}
filedir_outputs_roof_zoom_plot = {"out/" + filedir: "plt/" + filedir + "_roof_zoom_plot.png" for filedir in filedir_inputs}
filedir_inputs = ["out/" + filedir for filedir in filedir_inputs]
filedir_inputs = {filedir: os.listdir(filedir+"/") for filedir in filedir_inputs}

for filedir_input in filedir_inputs:
	# Plot bandwidth
	has_memory_data = False
	if any("memory" in x for x in filedir_inputs[filedir_input]):
		has_memory_data = True
		filedir = filedir_input + '/' + [s for s in filedir_inputs[filedir_input] if "memory" in s][0] + '/'
		filelist = [filedir + filename for filename in os.listdir(filedir)]
		threads_compact = []
		threads_scatter = []
		bytesek_compact = []
		bytesek_scatter = []

		for filename in filelist:
			with open(filename, 'r') as f:
				lines = f.readlines()
				lines = [line.split() for line in lines]
				if "compact" in filename:
				# Format for GByte/s
					threads_compact += [int(lines[0][0])]
					bytesek_compact += [float(lines[4][0])/1000]
				else:
					threads_scatter += [int(lines[0][0])]
					bytesek_scatter += [float(lines[4][0])/1000]

		plt.scatter(threads_compact, bytesek_compact, c='b', label='Compact')
		plt.scatter(threads_scatter, bytesek_scatter, c='g', label='Scatter')
		plt.legend(loc=4)
		plt.xlabel('Threads')
		plt.ylabel('Bandwidth [GB/s]')
		plt.ylim(bottom=0)
		plt.savefig(filedir_outputs_mem[filedir_input], bbox_inches='tight')
		plt.clf()

	if any("compute" in x for x in filedir_inputs[filedir_input]):
		filedir = filedir_input + '/' + [s for s in filedir_inputs[filedir_input] if "compute" in s][0] + '/'
		filelist = [filedir + filename for filename in os.listdir(filedir)]
		threads_compact = []
		threads_scatter = []
		flopsek_compact = []
		flopsek_scatter = []

		for filename in filelist:
			with open(filename, 'r') as f:
				lines = f.readlines()
				lines = [line.split() for line in lines]
				# Format for GFLOP/s
				if "compact" in filename:
					threads_compact += [int(lines[0][0])]
					flopsek_compact += [float(lines[4][0])/1000]
				else:
					threads_scatter += [int(lines[0][0])]
					flopsek_scatter += [float(lines[4][0])/1000]

		# Plot flop/s
		plt.scatter(threads_compact, flopsek_compact, c='b', label='Compact')
		plt.scatter(threads_scatter, flopsek_scatter, c='g', label='Scatter')
		plt.legend(loc='upper left')
		plt.xlabel('Threads')
		plt.ylabel('GFLOP/s')
		plt.ylim(bottom=0)
		plt.savefig(filedir_outputs_flops[filedir_input], bbox_inches='tight')
		plt.clf()

		# Plot roofline
		if has_memory_data:
			fvm_data = {}
			fvm_x = []
			fvm_y = []
			fvm_labels = []
			# Use FVM-MAP values for comparison
			if is_fvm_map:
				with open('../fvm-map/out/modified/benchmark.txt', 'r') as f:
					lines = [line.strip().split() for line in f.readlines()]
					for line in lines:
						filename = line[0]
						vertices_init = int(line[0].split('-')[-1][:-5])
						p = int(line[2])
						t = int(line[3])
						vertices = int(line[4])
						elements = int(line[5])
						flopbyte = float(line[14])
						# Should be divided by t from result.py in fvm_map
						flops_K = float(line[15]) / t
						flops_G = flops_K / 1000000
						label = 'V:{},R:{},T:{}'.format(vertices_init, p, t)
						fvm_data[filename] = [flopbyte, flops_G, label]
				n_data = len(fvm_data)
				fvm_data = sorted(fvm_data.values(), key=lambda k: k[1], reverse=True)[:n_data]
				for data in fvm_data:
					fvm_x += [data[0]]
					fvm_y += [data[1]]
					fvm_labels += [data[2]]

			# Rooftop values
			flop_roof_G = max(flopsek_compact + flopsek_scatter)
			byte_roof_G = max(bytesek_compact + bytesek_scatter)

			# Arithmetic intensity values
			ai_upper = 137/840
			ai_lower = 18/176
			ai_lower_1r = 191/1486

			# Plot zoomed in roof
			xs = np.linspace(0, 0.3, 100)
			bandwidth_line = byte_roof_G * xs
			plt.plot(xs, bandwidth_line, '-', label="Peak memory bandwidth ({:.2f} GB/s)".format(byte_roof_G))
			plt.xlabel('Operational intensity [FLOP/Byte]')
			plt.vlines(x=ai_upper, ymin=0, ymax=ai_upper*byte_roof_G, color='tab:blue', linestyle='--', label='Max AI: 0.1631')
			plt.vlines(x=ai_lower_1r, ymin=0, ymax=ai_lower_1r*byte_roof_G, color='tab:orange', linestyle='--', label='Min AI for 1R: 0.1277')
			plt.vlines(x=ai_lower, ymin=0, ymax=ai_lower*byte_roof_G, color='tab:green', linestyle='--', label='Min AI: 0.1023')
			plt.plot(fvm_x, fvm_y, 'o', color='g', label="Measured GFLOP/s")
			plt.xlabel('Arithmetic intensity [FLOP/Byte]')
			plt.ylabel('Attainable GFLOP/s')
			plt.ylim(bottom=0)
			handles,labels = plt.gca().get_legend_handles_labels()
			handles = [handles[0]] + handles[-3:] + [handles[1]]
			labels = [labels[0]] + labels[-3:] + [labels[1]]
			plt.legend(handles, labels)
			plt.savefig(filedir_outputs_roof_zoom_plot[filedir_input], bbox_inches='tight')
			plt.clf()

			# Plot zoomed in roof without fvm plot
			xs = np.linspace(0, 0.3, 100)
			bandwidth_line = byte_roof_G * xs
			plt.vlines(x=ai_upper, ymin=0, ymax=ai_upper*byte_roof_G, color='tab:blue', linestyle='--', label='Max AI: 0.1631')
			plt.vlines(x=ai_lower_1r, ymin=0, ymax=ai_lower_1r*byte_roof_G, color='tab:orange', linestyle='--', label='Min AI for 1R: 0.1277')
			plt.vlines(x=ai_lower, ymin=0, ymax=ai_lower*byte_roof_G, color='tab:green', linestyle='--', label='Min AI: 0.1023')
			plt.plot(xs, bandwidth_line, '-', label="Peak memory bandwidth ({:.2f} GB/s)".format(byte_roof_G))
			plt.legend()
			plt.xlabel('Arithmetic intensity [FLOP/Byte]')
			plt.ylabel('Attainable GFLOP/s')
			plt.ylim(bottom=0)
			plt.savefig(filedir_outputs_roof_zoom[filedir_input], bbox_inches='tight')
			plt.clf()

			# Use MKL benchmark
			if is_mkl:
				mkl_dir = '../mkl_benchmark/out/'
				mkl_files = [mkl_dir + mkl_file for mkl_file in os.listdir(mkl_dir)]
				mkl_flops_max_G = 0.0
				for mkl_file in mkl_files:
					with open(mkl_file, 'r') as f:
						lines = [line.strip().split() for line in f.readlines()]
						tmp = lines[20]
						mkl_flops_G = max([float(line[0]) for line in lines if line[1] == '[GFLOP/s]'])
						mkl_flops_max_G = mkl_flops_G if mkl_flops_G > mkl_flops_max_G else mkl_flops_max_G
				flop_roof_G = mkl_flops_max_G

			cross_x = flop_roof_G / byte_roof_G
			max_x = max(max(fvm_x)*2, cross_x*3) if is_fvm_map else cross_x*3
			max_y = max(max(fvm_y)*1.3, flop_roof_G*1.2) if is_fvm_map else flop_roof_G*1.2
			plt.plot([0, cross_x],[0, flop_roof_G])
			plt.hlines(y=flop_roof_G, xmin=cross_x, xmax=max_x)
			#if is_fvm_map:
				#for i in range(n_data):
					#plt.plot([fvm_x[i]], [fvm_y[i]], 'o', label=fvm_labels[i])
				#plt.legend(loc=4)
			plt.text(cross_x-0.2, flop_roof_G*1.02, 'Peak floating point performance')
			plt.text(cross_x/3, flop_roof_G/2, 'Peak memory bandwidth ({:.2f} GB/s)'.format(byte_roof_G))
			plt.xlabel('Arithmetic intensity [FLOP/Byte]')
			plt.ylabel('Attainable GFLOP/s')
			plt.ylim(bottom=0, top=max_y)
			plt.xlim(left=0, right=max_x)
			plt.savefig(filedir_outputs_roof[filedir_input], bbox_inches='tight')
			plt.clf()

			cross_x = flop_roof_G / byte_roof_G
			max_x = max(max(fvm_x)*2, cross_x*3) if is_fvm_map else cross_x*3
			max_y = max(max(fvm_y)*1.3, flop_roof_G*1.2) if is_fvm_map else flop_roof_G*1.2
			plt.plot([0, cross_x],[0, flop_roof_G])
			plt.hlines(y=flop_roof_G, xmin=cross_x, xmax=max_x)
			plt.text(cross_x-0.2, flop_roof_G*1.02, 'Peak floating point performance')
			plt.gca().axes.xaxis.set_ticks([])
			plt.gca().axes.yaxis.set_ticks([])
			plt.text(cross_x/3, flop_roof_G/2, 'Peak memory bandwidth')
			plt.xlabel('Arithmetic intensity [FLOP/Byte]')
			plt.ylabel('Attainable FLOP/s')
			plt.ylim(bottom=0, top=max_y)
			plt.xlim(left=0, right=max_x)
			plt.savefig(filedir_outputs_roof[filedir_input][:-4] + '_empty.png', bbox_inches='tight')
			plt.clf()

			# Write data to fvm-map
			#with open('../fvm-map/out/input/prediction_roof.txt', 'w') as f:
				#print(byte_roof_G, 'GByte/s', file=f)
				#print(flop_roof_G, 'GFLOP/s', file=f)
