#include "memory_test.h"

#define BILLION 1e9
#define MILLION 1e6
#define random(max_random) ((real_t) rand()/(real_t) (RAND_MAX)) * (max_random);
#define pow2(x) ((x)*(x))
#define sched
//schedule(static,N)
//schedule(dynamic,N)
//schedule(guided,N)


static int
	size,
	rank;

static int_t
	n_tests_mem = 25,
	len_array = 1e6,
	n_array = 4,
	n_element_array = 2,
	test_index = 555,
	max_random = 1,
	max_random_int = 1000;

static real_t
	start = 0.0,
	end = 0.0,
	*time_memory_real = NULL,
	*time_memory_tri = NULL,
	*time_memory_tri_int = NULL,
	*time_memory_pointer = NULL,
	*data_array0 = NULL,
	*data_array1 = NULL,
	*data_array2 = NULL,
	*data_array3 = NULL,
	*data_arrays[4] = {NULL, NULL, NULL, NULL};

static triangle_t
	*elements0 = NULL,
	*elements1 = NULL,
	*elements_array[2] = {NULL, NULL};


static void init ( void )
{
	time_memory_real = malloc (n_tests_mem * sizeof(real_t));
	time_memory_tri = malloc (n_tests_mem * sizeof(real_t));
	time_memory_tri_int = malloc (n_tests_mem * sizeof(real_t));
	time_memory_pointer = malloc (n_tests_mem * sizeof(real_t));

	data_array0 = malloc (len_array * sizeof(real_t));
	data_array1 = malloc (len_array * sizeof(real_t));
	data_array2 = malloc (len_array * sizeof(real_t));
	data_array3 = malloc (len_array * sizeof(real_t));
	data_arrays[0] = data_array0;
	data_arrays[1] = data_array1;
	data_arrays[2] = data_array2;
	data_arrays[3] = data_array3;

	elements0 = malloc (len_array * sizeof(triangle_t));
	elements1 = malloc (len_array * sizeof(triangle_t));
	elements_array[0] = elements0;
	elements_array[1] = elements1;

	for (int_t i=0; i<n_array; i++)
		#pragma omp parallel for sched
		for (int_t j=0; j<len_array; j++)
			data_arrays[i][j] = random(max_random);
	
	for (int_t i=0; i<n_element_array; i++) {
		for (int_t j=0; j<len_array; j++) {
			triangle_t *element = &elements_array[i][j];
			element->id = random(max_random_int);
			element->id_local = random(max_random_int);
			element->v[0] = random(max_random_int);
			element->v[1] = random(max_random_int);
			element->v[2] = random(max_random_int);
			element->c[0] = random(max_random);
			element->c[1] = random(max_random);
			element->c[2] = random(max_random);
			element->area = random(max_random);
			element->n_neighbors = random(max_random_int);
			element->neighbors_id[0] = random(max_random_int);
			element->neighbors_id[1] = random(max_random_int);
			element->neighbors_id[2] = random(max_random_int);
			element->neighbors_id_local[0] = random(max_random_int);
			element->neighbors_id_local[1] = random(max_random_int);
			element->neighbors_id_local[2] = random(max_random_int);
		}
	}
}


static void finalize ( void )
{
	free(time_memory_real);
	free(time_memory_tri);
	free(time_memory_tri_int);
	free(time_memory_pointer);
	free(data_array0);
	free(data_array1);
	free(data_array2);
	free(data_array3);
	free(elements0);
	free(elements1);
}


static void print_data_size ( void )
{
	int_t size_int_t = sizeof(int_t);
	int_t size_real_t = sizeof(real_t);
	int_t size_triangle_t = sizeof(triangle_t);
	int_t size_int_t_p = sizeof(int_t *);
	int_t size_real_t_p = sizeof(real_t *);
	int_t size_triangle_t_p = sizeof(triangle_t *);
	printf("\nSize of:\n"
		   "int_t = %ld bytes\n"
		   "real_t = %ld bytes\n"
		   "triangle_t = %ld bytes\n"
		   "*int_t = %ld bytes\n"
		   "*real_t = %ld bytes\n"
		   "*triangle_t = %ld bytes\n\n",
		   size_int_t, size_real_t, size_triangle_t,
		   size_int_t_p, size_real_t_p, size_triangle_t_p
	);
}


static void print_result_memory ( int_t mem_type, int_t bytes )
{
	real_t *time_samples;
	switch(mem_type) {
		case 0:
			time_samples = time_memory_real;
			break;
		case 1:
			time_samples = time_memory_tri;
			break;
		case 2:
			time_samples = time_memory_tri_int;
			break;
		case 3:
			time_samples = time_memory_pointer;
	}

	real_t avg_time = 0.0;
	real_t var_time = 0.0;
	for (int_t i=0; i<n_tests_mem; i++) {
		avg_time += time_samples[i];
	}
	avg_time /= n_tests_mem;

	for (int_t i=0; i<n_tests_mem; i++) {
		var_time += pow2(time_samples[i] - avg_time);
	}
	var_time /= ( n_tests_mem - 1 );
	var_time = sqrt( var_time );

	real_t reduced_avg_time = 0.0;
	real_t reduced_var_time = 0.0;
	MPI_Reduce(&avg_time, &reduced_avg_time, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
	MPI_Reduce(&var_time, &reduced_var_time, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
	MPI_Barrier(MPI_COMM_WORLD);
	reduced_avg_time /= size;
	reduced_var_time /= size;

	if (rank==0) {
		printf("%lf +- %lf microsec for array access\n", reduced_avg_time*MILLION, reduced_var_time*MILLION);
		printf("%lf Mbyte/s\n", (bytes / reduced_avg_time)/MILLION);
	}
}


static void warm_up ( void )
{
	real_t
		data0 = data_array0[test_index],
		data1 = data_array1[test_index],
		data2 = data_array2[test_index];
	#pragma omp parallel for sched
	for (int_t i=0; i<len_array; i++) {
		data_array0[i] = data_array1[i];
		elements0[i].id = elements1[i].id;
		data0 = data1 + data2;
		data0 = data0 * data1;
		data0 = data0 / data2;
	}
}


static void memory_real_test ( void )
{
	for (int_t i=0; i<n_tests_mem; i++) {
		start = MPI_Wtime();
		#pragma omp parallel for sched
		for (int_t j=0; j<len_array; j++) {
			data_array0[j] = data_array1[j];
		}
		end = MPI_Wtime();
		time_memory_real[i] = end - start;
	}
	if (rank==0)
		printf("real_t test:\n");
	print_result_memory(0, 2 * sizeof(real_t) * len_array);
}


static void memory_tri_test ( void )
{
	for (int_t i=0; i<n_tests_mem; i++) {
		start = MPI_Wtime();
		#pragma omp parallel for sched
		for (int_t j=0; j<len_array; j++) {
			elements0[j] = elements1[j];
		}
		end = MPI_Wtime();
		time_memory_tri[i] = end - start;
	}
	if (rank==0)
		printf("triangle_t test:\n");
	print_result_memory(1, 2 * sizeof(triangle_t) * len_array);
}


static void memory_tri_int_test ( void )
{
	for (int_t i=0; i<n_tests_mem; i++) {
		int_t nidx = 0;
		start = MPI_Wtime();
		#pragma omp parallel for sched
		for (int_t j=0; j<len_array; j++) {
			nidx = elements0[j].neighbors_id_local[0];
		}
		end = MPI_Wtime();
		time_memory_tri_int[i] = end - start;
		nidx++;
	}
	if (rank==0)
		printf("int_t from triangle_t test:\n");
	print_result_memory(2, (sizeof(triangle_t *) + sizeof(int_t)) * len_array);
}


static void memory_pointer_test ( void )
{
	triangle_t *element = NULL;
	for (int_t i=0; i<n_tests_mem; i++) {
		start = MPI_Wtime();
		#pragma omp parallel for sched
		for (int_t j=0; j<len_array; j++) {
			element = &(elements0[j]);
		}
		end = MPI_Wtime();
		time_memory_pointer[i] = end - start;
	}
	element->id++;
	if (rank==0)
		printf("pointer of triangle_t test:\n");
	print_result_memory(3, sizeof(triangle_t *) * len_array);
}


void memory_test ( void )
{
	MPI_Init(NULL, NULL);
	MPI_Comm_size(MPI_COMM_WORLD, &size);
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);

	init();

	if (rank==0) {
		printf("%d %d (threads, processes)\n", omp_get_max_threads(), size);
		printf("%ld %ld (array length, tests)\n", len_array, n_tests_mem);
	}
	MPI_Barrier(MPI_COMM_WORLD);

	warm_up();
	memory_real_test();
	//memory_tri_test();
	//memory_tri_int_test();
	//memory_pointer_test();
	//print_data_size();

	finalize();
	MPI_Finalize();
}

