#ifndef TLHASH_H
#define TLHASH_H
#include <stddef.h>
#include <stdint.h>

/* Type signature of admissible hash functions */
typedef uint32_t (*tlhash_function_t)(void *, size_t );

/* Data structures for elements and tables of elements */
typedef struct el {
    void *key, *value;
    size_t key_length;
    struct el *next;
} tlhash_element_t;

typedef struct {
    size_t n_buckets, size;
    tlhash_element_t **buckets;
    tlhash_function_t hash_function;
} tlhash_t;


/* Programming interface */
int tlhash_init ( tlhash_t *tab, size_t n_buckets );
int tlhash_finalize ( tlhash_t *tab );
int tlhash_insert ( tlhash_t *tab, void *key, size_t keylen, void *val );
int tlhash_lookup ( tlhash_t *tab, void *key, size_t keylen, void **val );
int tlhash_remove ( tlhash_t *tab, void *key, size_t key_length );
size_t tlhash_size ( tlhash_t *tab );
void tlhash_keys ( tlhash_t *tab, void **keys );
void tlhash_values ( tlhash_t *tab, void **values );
int tlhash_set_hash_function ( int hash_function );


/* Error code definitions */
enum {
    TLHASH_SUCCESS = 0,    /* Success                    */
    TLHASH_ENOMEM  = 1,    /* Unable to allocate memory  */
    TLHASH_ENOENT  = 2,    /* No such table entry        */
    TLHASH_EEXIST  = 3,    /* Table entry already exists */
    TLHASH_EINVAL  = 4     /* Invalid argument value     */
};


/* Selection of hash functions */
enum {
    TLHASH_CRC   = 0,
    TLHASH_ADLER = 1
};
#endif
