import os
import numpy as np
from matplotlib import pyplot as plt

dir_out = 'out/'
dir_structure = dir_out + 'structure/'
dir_raw = dir_out + 'raw/'
dir_modified = dir_out + 'modified/'
dir_plot = dir_out + 'plot/'
dir_input = dir_out + 'input/'
dir_plot_prediction = dir_plot + 'prediction/'

structure_files = os.listdir(dir_structure)
raw_files = os.listdir(dir_raw)

structure_data = {}
raw_data = {}
raw_data_details = {}

# Constants
n_points_plot = 2
#labels_limit_t = [6, 4, 3]
labels_limit_t = [6, 4]
font_size = 15
ranks = 1
threads = 1
xlabel = "Nodes"
max_proc = 24

plt.rcParams.update({'font.size': font_size})

for structure_file in structure_files:
	with open(dir_structure + structure_file, 'r') as f:
		lines = [line.split() for line in f.readlines()[:6]]
		vertices_init = int(structure_file.split('v')[0])
		vertices = int(lines[0][0])
		elements = int(lines[0][1])
		max_upper = int(lines[1][0])
		max_lower = int(lines[1][1])
		n_local = int(lines[1][2])
		n_neuman = int(lines[1][3])
		flop_tot = int(lines[2][0])
		mem_tot = int(lines[3][0])
		flopbyte = float(lines[4][0])
		p = int(lines[5][0])
		structure_data[structure_file] = [vertices_init, vertices, elements, max_upper, max_lower, n_local, n_neuman, flop_tot, mem_tot, flopbyte, p]

for raw_file in raw_files:
	with open(dir_raw + raw_file, 'r') as f:
		lines = [line.split() for line in f.readlines()]
		nodes = int(raw_file.split('n')[0])
		p = int(lines[1][0])
		t = int(lines[2][0])
		vertices = int(lines[3][0])
		elements = int(lines[4][0])
		samples = int(lines[5][0])
		vertices_init = int(raw_file.split('-')[-1][:-5])
		structure_file = str(vertices_init) + 'v-' + str(p) + 'p.txt'
		raw_data[raw_file] = [structure_file, nodes, p, t, vertices_init, vertices, elements, samples]

		calc_avg, calc_avg_p1, calc_avg_p2, calc_avg_p3, halo_avg = [0.0]*p, [0.0]*p, [0.0]*p, [0.0]*p, [0.0]*p
		calc_var, calc_var_p1, calc_var_p2, calc_var_p3, halo_var = [0.0]*p, [0.0]*p, [0.0]*p, [0.0]*p, [0.0]*p
		for x in range(p):
			index = 7 + x * 7
			rank = int(lines[index][1])
			calc_avg[rank] = float(lines[index+1][0])
			calc_avg_p1[rank] = float(lines[index+2][0])
			calc_avg_p2[rank] = float(lines[index+3][0])
			calc_avg_p3[rank] = float(lines[index+4][0])
			halo_avg[rank] = float(lines[index+5][0])
			calc_var[rank] = float(lines[index+1][2])
			calc_var_p1[rank] = float(lines[index+2][2])
			calc_var_p2[rank] = float(lines[index+3][2])
			calc_var_p3[rank] = float(lines[index+4][2])
			halo_var[rank] = float(lines[index+5][2])
		raw_data_details[raw_file] = [calc_avg, calc_avg_p1, calc_avg_p2, calc_avg_p3, halo_avg]
		raw_data_details[raw_file] += [calc_var, calc_var_p1, calc_var_p2, calc_var_p3, halo_var]

		calc_avg_reduced = sum(calc_avg) / p
		calc_avg_p1_reduced = sum(calc_avg_p1) / p
		calc_avg_p2_reduced = sum(calc_avg_p2) / p
		calc_avg_p3_reduced = sum(calc_avg_p3) / p
		halo_avg_reduced = sum(halo_avg) / p
		raw_data[raw_file] += [calc_avg_reduced, calc_avg_p1_reduced, calc_avg_p2_reduced, calc_avg_p3_reduced, halo_avg_reduced]

# Data used for benchmarking
with open(dir_modified + "benchmark.txt", 'w') as f:
	for raw_file in raw_files:
		data = raw_data[raw_file]
		structure_file = data[0]
		nodes = data[1]
		p = data[2]
		t = data[3]
		vertices = data[5]
		elements = data[6]
		calc_avg_reduced = data[8]
		halo_avg_reduced = data[12]

		structure = structure_data[structure_file]
		max_upper = structure[3]
		max_lower = structure[4]
		n_local = structure[5]
		n_neuman = structure[6]
		flop_tot = structure[7]
		flopbyte = structure[9]
		flops = flop_tot / calc_avg_reduced

		print(raw_file, nodes, p, t, vertices, elements, max_upper, max_lower, n_local, n_neuman, calc_avg_reduced, 'ms', halo_avg_reduced, 'ms', flopbyte, flops, ('KFLOP/s'), file=f)

#bandwidth = 43.237821694 * 1e6
bandwidth = 1e6
def time_prediction_strong(V, R, T):
	return ( 210*V/R + 285.6*np.sqrt(V)/R + 49.28*np.sqrt(V) ) * 8 / (T * bandwidth)

def time_prediction_p1_strong(V, R, T):
	return ( (8.5*np.sqrt(V)*1.12/R) * 8 + (8.5*np.sqrt(V)*1.12/R + 2*np.sqrt(V)*1.12+ 2*V/R) * 22 ) * 8 / (T * bandwidth)

def time_prediction_p2_strong(V, R, T):
	return ( (2*V/R) * 47 ) * 8 / (T * bandwidth)

def time_prediction_p3_strong(V, R, T):
	return ( (2*V/R) * 36 ) * 8 / (T * bandwidth)

def time_prediction_weak(V, R, T):
	return ( 210*V*T + 285.6*np.sqrt(V*R*T)/R + 49.28*np.sqrt(V*R*T) ) * 8 / (T * bandwidth)

def time_prediction_p1_weak(V, R, T):
	return ( (8.5*np.sqrt(V*R*T)*1.12/R) * 8 + (8.5*np.sqrt(V*R*T)*1.12/R + 2*np.sqrt(V*R*T)*1.12+ 2*V*R*T/R) * 22 ) * 8 / (T * bandwidth)

def time_prediction_p2_weak(V, R, T):
	return ( (2*V*R*T/R) * 47 ) * 8 / (T * bandwidth)

def time_prediction_p3_weak(V, R, T):
	return ( (2*V*R*T/R) * 36 ) * 8 / (T * bandwidth)

def time_prediction_strong_triangles(neu, halo, triangles, R, T):
	return ( 240*neu/R + 352*halo + 840*triangles/R ) / (T * bandwidth)

# Constants
xs_proc = np.linspace(1, 24, 100)
xs_v = np.linspace(2000, 20000, 100)
fixed_v_strong = [20000]
fixed_v_weak = [2000]
fixed_t = 1
fixed_r = 1
xlabel_proc = 'Number of parallel processes'
xlabel_v = 'Vertices'
ylabel_time = 'Runtime prediction [ms]'
ylabel_operations = 'Memory operations [MB]'
ylim_top_weak = time_prediction_weak(fixed_v_weak[-1], fixed_r, fixed_t) * 1.2

# Plot formula for strong time prediction for any maps
v_strong = fixed_v_strong[0]
triangles_max = 2*v_strong-4
neu_max = 2*triangles_max
halo_max = triangles_max/2
triangles_min = v_strong-2
neu_min = 3
halo_min = 0
formula_ranks_max = time_prediction_strong_triangles(neu_max, halo_max, triangles_max, xs_proc, fixed_t)
formula_threads_max = time_prediction_strong_triangles(neu_max, halo_max, triangles_max, fixed_r, xs_proc)
formula_ranks_min = time_prediction_strong_triangles(neu_min, halo_min, triangles_min, xs_proc, fixed_t)
formula_threads_min = time_prediction_strong_triangles(neu_min, halo_min, triangles_min, fixed_r, xs_proc)
plt.plot(xs_proc, formula_ranks_max, '-', label='Upper bound: Ranks')
plt.plot(xs_proc, formula_threads_max, '-', label='Upper bound: Threads')
plt.plot(xs_proc, formula_ranks_min, '--', label='Lower bound: Ranks', linewidth=3)
plt.plot(xs_proc, formula_threads_min, '-', label='Lower bound: Threads')
plt.legend()
plt.xlabel(xlabel_proc)
plt.ylabel(ylabel_operations)
plt.ylim(bottom=0)
plt.savefig(dir_plot_prediction + 'formula_general_time_strong.png', bbox_inches='tight')
plt.clf()

# Plot formula for weak time prediction for any maps
v_weak = fixed_v_weak[0]*xs_proc
triangles_max = 2*v_weak-4
neu_max = 4*v_weak-8
halo_max = v_weak-2
triangles_min = v_weak-2
neu_min = 3
halo_min = 0
formula_ranks_max = time_prediction_strong_triangles(neu_max, halo_max, triangles_max, xs_proc, fixed_t)
formula_threads_max = time_prediction_strong_triangles(neu_max, halo_max, triangles_max, fixed_r, xs_proc)
formula_ranks_min = time_prediction_strong_triangles(neu_min, halo_min, triangles_min, xs_proc, fixed_t)
formula_threads_min = time_prediction_strong_triangles(neu_min, halo_min, triangles_min, fixed_r, xs_proc)
plt.plot(xs_proc, formula_ranks_max, '-', label='Upper bound: Ranks')
plt.plot(xs_proc, formula_threads_max, '-', label='Upper bound: Threads')
plt.plot(xs_proc, formula_ranks_min, '--', label='Lower bound: Ranks', linewidth=3)
plt.plot(xs_proc, formula_threads_min, '-', label='Lower bound: Threads')
plt.legend()
plt.xlabel(xlabel_proc)
plt.ylabel(ylabel_operations)
plt.ylim(bottom=0)
plt.savefig(dir_plot_prediction + 'formula_general_time_weak.png', bbox_inches='tight')
plt.clf()

# Plot formula for strong time prediction
v_strong = fixed_v_strong[0]
triangles_max = 2*v_strong-4
neu_max = 2*triangles_max
halo_max = triangles_max/2
triangles_min = v_strong-2
neu_min = 3
halo_min = 0
formula_ranks_max = time_prediction_strong_triangles(neu_max, halo_max, triangles_max, xs_proc, fixed_t)
formula_threads_min = time_prediction_strong_triangles(neu_min, halo_min, triangles_min, fixed_r, xs_proc)
plt.plot(xs_proc, formula_ranks_max, ':', color='tab:brown', label='Upper bound')
plt.plot(xs_proc, formula_threads_min, ':', color='tab:gray', label='Lower bound')
for v in fixed_v_strong:
	formula_ranks = time_prediction_strong(v, xs_proc, fixed_t)
	formula_threads = time_prediction_strong(v, fixed_r, xs_proc)
	if v == fixed_v_strong[0]:
		plt.plot(xs_proc, formula_ranks, '--', color='tab:blue', label='Approx: Ranks', linewidth=3)
		plt.plot(xs_proc, formula_threads, '-', color='tab:orange', label='Approx: Threads')
	else:
		plt.plot(xs_proc, formula_ranks, '--', color='tab:blue', linewidth=3)
		plt.plot(xs_proc, formula_threads, '-', color='tab:orange')
plt.legend()
plt.xlabel(xlabel_proc)
plt.ylabel(ylabel_operations)
plt.ylim(bottom=0)
plt.savefig(dir_plot_prediction + 'formula_time_strong.png', bbox_inches='tight')
plt.clf()

# Plot formula for strong p1,p2,p3 prediction
for v in fixed_v_strong:
	formula_ranks = time_prediction_strong(v, xs_proc, fixed_t)
	formula_ranks_p1 = time_prediction_p1_strong(v, xs_proc, fixed_t)
	formula_ranks_p2 = time_prediction_p2_strong(v, xs_proc, fixed_t)
	formula_ranks_p3 = time_prediction_p3_strong(v, xs_proc, fixed_t)
	formula_ranks_sum = formula_ranks_p1 + formula_ranks_p2 + formula_ranks_p3
	formula_threads = time_prediction_strong(v, fixed_r, xs_proc)
	formula_threads_p1 = time_prediction_p1_strong(v, fixed_r, xs_proc)
	formula_threads_p2 = time_prediction_p2_strong(v, fixed_r, xs_proc)
	formula_threads_p3 = time_prediction_p3_strong(v, fixed_r, xs_proc)
	formula_threads_sum = formula_threads_p1 + formula_threads_p2 + formula_threads_p3
	if v == fixed_v_strong[0]:
		#plt.plot(xs_proc, formula_ranks, '-', color='tab:pink', label='Ranks')
		plt.plot(xs_proc, formula_ranks_p1, '-', color='tab:blue', label='P1 Ranks')
		plt.plot(xs_proc, formula_ranks_p2, '-', color='tab:orange', label='P2 Ranks')
		plt.plot(xs_proc, formula_ranks_p3, '-', color='tab:green', label='P3 Ranks')
		#plt.plot(xs_proc, formula_ranks_sum, '-', color='tab:gray', label='Sum Ranks')
		#plt.plot(xs_proc, formula_threads, '-', color='tab:olive', label='Threads')
		plt.plot(xs_proc, formula_threads_p1, '-', color='tab:red', label='P1 Threads')
		plt.plot(xs_proc, formula_threads_p2, '-', color='tab:purple', label='P2 Threads')
		plt.plot(xs_proc, formula_threads_p3, '-', color='tab:brown', label='P3 Threads')
		#plt.plot(xs_proc, formula_threads_sum, '-', color='tab:cyan', label='Sum Threads')
	else:
		#plt.plot(xs_proc, formula_ranks, '-', color='tab:pink')
		plt.plot(xs_proc, formula_ranks_p1, '-', color='tab:blue')
		plt.plot(xs_proc, formula_ranks_p2, '-', color='tab:orange')
		plt.plot(xs_proc, formula_ranks_p3, '-', color='tab:green')
		#plt.plot(xs_proc, formula_ranks_sum, '-', color='tab:gray')
		#plt.plot(xs_proc, formula_threads, '-', color='tab:olive')
		plt.plot(xs_proc, formula_threads_p1, '-', color='tab:red')
		plt.plot(xs_proc, formula_threads_p2, '-', color='tab:purple')
		plt.plot(xs_proc, formula_threads_p3, '-', color='tab:brown')
		#plt.plot(xs_proc, formula_threads_sum, '-', color='tab:cyan')
plt.legend()
plt.xlabel(xlabel_proc)
plt.ylabel(ylabel_operations)
plt.ylim(bottom=0)
plt.savefig(dir_plot_prediction + 'formula_parts_strong.png', bbox_inches='tight')
plt.clf()

# Plot formula for weak time prediction
v_weak = fixed_v_weak[0]*xs_proc
triangles_max = 2*v_weak-4
neu_max = 4*v_weak-8
halo_max = v_weak-2
triangles_min = v_weak-2
neu_min = 3
halo_min = 0
formula_ranks_max = time_prediction_strong_triangles(neu_max, halo_max, triangles_max, xs_proc, fixed_t)
formula_threads_min = time_prediction_strong_triangles(neu_min, halo_min, triangles_min, fixed_r, xs_proc)
plt.plot(xs_proc, formula_ranks_max, ':', color='tab:brown', label='Upper bound')
plt.plot(xs_proc, formula_threads_min, ':', color='tab:gray', label='Lower bound')
for v in fixed_v_weak:
	formula_ranks = time_prediction_weak(v, xs_proc, fixed_t)
	formula_threads = time_prediction_weak(v, fixed_r, xs_proc)
	if v == fixed_v_weak[0]:
		plt.plot(xs_proc, formula_ranks, '--', color='tab:blue', label='Approx: Ranks', linewidth=2)
		plt.plot(xs_proc, formula_threads, '-', color='tab:orange', label='Approx: Threads')
	else:
		plt.plot(xs_proc, formula_ranks, '--', color='tab:blue', linewidth=3)
		plt.plot(xs_proc, formula_threads, '-', color='tab:orange')
plt.legend()
plt.xlabel(xlabel_proc)
plt.ylabel(ylabel_operations)
plt.ylim(bottom=0)
plt.savefig(dir_plot_prediction + 'formula_time_weak.png', bbox_inches='tight')
plt.clf()

# Plot formula for weak p1,p2,p3 prediction
for v in fixed_v_weak:
	formula_ranks = time_prediction_weak(v, xs_proc, fixed_t)
	formula_ranks_p1 = time_prediction_p1_weak(v, xs_proc, fixed_t)
	formula_ranks_p2 = time_prediction_p2_weak(v, xs_proc, fixed_t)
	formula_ranks_p3 = time_prediction_p3_weak(v, xs_proc, fixed_t)
	formula_ranks_sum = formula_ranks_p1 + formula_ranks_p2 + formula_ranks_p3
	formula_threads = time_prediction_weak(v, fixed_r, xs_proc)
	formula_threads_p1 = time_prediction_p1_weak(v, fixed_r, xs_proc)
	formula_threads_p2 = time_prediction_p2_weak(v, fixed_r, xs_proc)
	formula_threads_p3 = time_prediction_p3_weak(v, fixed_r, xs_proc)
	formula_threads_sum = formula_threads_p1 + formula_threads_p2 + formula_threads_p3
	if v == fixed_v_weak[0]:
		#plt.plot(xs_proc, formula_ranks, '-', color='tab:pink', label='Ranks')
		plt.plot(xs_proc, formula_ranks_p1, '-', color='tab:blue', label='P1 Ranks')
		plt.plot(xs_proc, formula_ranks_p2, '-', color='tab:orange', label='P2 Ranks')
		plt.plot(xs_proc, formula_ranks_p3, '-', color='tab:green', label='P3 Ranks')
		#plt.plot(xs_proc, formula_ranks_sum, '-', color='tab:gray', label='Sum Ranks')
		#plt.plot(xs_proc, formula_threads, '-', color='tab:olive', label='Threads')
		plt.plot(xs_proc, formula_threads_p1, '-', color='tab:red', label='P1 Threads')
		plt.plot(xs_proc, formula_threads_p2, '-', color='tab:purple', label='P2 Threads')
		plt.plot(xs_proc, formula_threads_p3, '-', color='tab:brown', label='P3 Threads')
		#plt.plot(xs_proc, formula_threads_sum, '-', color='tab:cyan', label='Sum Threads')
	else:
		#plt.plot(xs_proc, formula_ranks, '-', color='tab:pink')
		plt.plot(xs_proc, formula_ranks_p1, '-', color='tab:blue')
		plt.plot(xs_proc, formula_ranks_p2, '-', color='tab:orange')
		plt.plot(xs_proc, formula_ranks_p3, '-', color='tab:green')
		#plt.plot(xs_proc, formula_ranks_sum, '-', color='tab:gray')
		#plt.plot(xs_proc, formula_threads, '-', color='tab:olive')
		plt.plot(xs_proc, formula_threads_p1, '-', color='tab:red')
		plt.plot(xs_proc, formula_threads_p2, '-', color='tab:purple')
		plt.plot(xs_proc, formula_threads_p3, '-', color='tab:brown')
		#plt.plot(xs_proc, formula_threads_sum, '-', color='tab:cyan')
plt.legend()
plt.xlabel(xlabel_proc)
plt.ylabel(ylabel_operations)
plt.ylim(bottom=0, top=ylim_top_weak)
plt.savefig(dir_plot_prediction + 'formula_parts_weak.png', bbox_inches='tight')
plt.clf()

# Plot formula for increasing V
formula_vertices_p1 = time_prediction_p1_strong(xs_v, fixed_r, fixed_t)
formula_vertices_p2 = time_prediction_p2_strong(xs_v, fixed_r, fixed_t)
formula_vertices_p3 = time_prediction_p3_strong(xs_v, fixed_r, fixed_t)
formula_vertices = time_prediction_strong(xs_v, fixed_t, fixed_r)
plt.plot(xs_v, formula_vertices_p1, '-', label='P1 proc')
plt.plot(xs_v, formula_vertices_p2, '-', label='P2 proc')
plt.plot(xs_v, formula_vertices_p3, '-', label='P3 proc')
plt.plot(xs_v, formula_vertices, '-', label='Sum proc')
xticks_v = [min(xs_v), xs_v[len(xs_v)//2], max(xs_v)]
plt.xticks(xticks_v)
plt.legend()
plt.xlabel(xlabel_v)
plt.ylabel(ylabel_operations)
plt.ylim(bottom=0)
plt.savefig(dir_plot_prediction + 'formula_vertices.png', bbox_inches='tight')
plt.clf()



########################
# Arithmetic intensity #
########################

def arithmetic_intensity(V, R):
	mem_op = ( 210*V/R + 285.6*np.sqrt(V)/R + 49.28*np.sqrt(V) ) * 8
	flop_op = (8.5*np.sqrt(V)*1.12/R + 2*np.sqrt(V)*1.12 + 2*V/R) * 18 + (2*V/R) * 70 + (2*V/R) * 49
	return flop_op / mem_op

def arithmetic_intensity2(V, R):
	mem_op = 8.5*np.sqrt(V)*1.12/R * 240 + np.sqrt(V)*1.12 * 352 + 2*V/R * 840
	flop_op = 8.5*np.sqrt(V)*1.12/R * 18 + np.sqrt(V)*1.12 * 36 + 2*V/R * 137
	return flop_op / mem_op

def arithmetic_intensity_triangle(neu, halo, T, R):
	mem_op = neu/R * 240 + halo * 352 + T/R * 840
	flop_op = neu/R * 18 + halo * 36 + T/R * 137
	return flop_op / mem_op

# Constants for arithmetic intensity
xs_v_ai = np.linspace(1, 200, 100)
xs_r_ai = np.linspace(1, 1000, 100)
fixed_v_ai = 200
fixed_r_ai = 1
xlabel_v_ai = 'Vertices'
xlabel_r_ai = 'Ranks'
ylabel_ai = 'Arithmetic intensity'
ylim_top_ai = 0.2

# Plot arithmetic intensity
formula_arithmetic_intensity = arithmetic_intensity(xs_v_ai, fixed_r_ai)
#formula_arithmetic_intensity2 = arithmetic_intensity2(xs_v_ai, fixed_r_ai)
formula_arithmetic_intensity2 = arithmetic_intensity_triangle(8.5*np.sqrt(xs_v_ai)*1.12, np.sqrt(xs_v_ai)*1.12, 2*xs_v_ai, fixed_r_ai)
plt.plot(xs_v_ai, formula_arithmetic_intensity, '-')
#plt.plot(xs_v_ai, formula_arithmetic_intensity2, '-', label='Arithmetic intensity 2')
plt.legend()
plt.xlabel(xlabel_v_ai)
plt.ylabel(ylabel_ai)
plt.ylim(bottom=0, top=ylim_top_ai)
plt.savefig(dir_plot_prediction + 'formula_arithmetic_intensity.png', bbox_inches='tight')
plt.clf()
print(arithmetic_intensity(20000, 1))

# Plot arithmetic intensity for ranks
formula_arithmetic_intensity = arithmetic_intensity(fixed_v_ai, xs_r_ai)
#formula_arithmetic_intensity2 = arithmetic_intensity2(fixed_v_ai, xs_r_ai)
formula_arithmetic_intensity2 = arithmetic_intensity_triangle(8.5*np.sqrt(fixed_v_ai)*1.12, np.sqrt(fixed_v_ai)*1.12, 2*fixed_v_ai, xs_r_ai)
plt.plot(xs_r_ai, formula_arithmetic_intensity, '-')
#plt.plot(xs_r_ai, formula_arithmetic_intensity2, '-', label='Arithmetic intensity 2')
plt.legend()
plt.xlabel(xlabel_r_ai)
plt.ylabel(ylabel_ai)
plt.ylim(bottom=0, top=ylim_top_ai)
plt.savefig(dir_plot_prediction + 'formula_arithmetic_intensity_ranks.png', bbox_inches='tight')
plt.clf()


# New constants
fixed_r = 1
xs_r = np.linspace(1, 200, 100)
fixed_T = 2 * 1e9
xs_neu = np.linspace(3, 2*fixed_T, 100)
xs_halo = np.linspace(0, fixed_T/2, 100)
xs_T = np.linspace(1, fixed_T, 100)

# Plot general arithmetic intensity ranks
formula_general_r = arithmetic_intensity_triangle(fixed_T, fixed_T, fixed_T, xs_r)
plt.plot(xs_r, formula_general_r, '-')
plt.xlabel('Ranks')
plt.ylabel(ylabel_ai)
plt.ylim(bottom=0, top=ylim_top_ai)
plt.savefig(dir_plot_prediction + 'formula_general_arithmetic_intensity_ranks.png', bbox_inches='tight')
plt.clf()

# Plot general arithmetic intensity neumann
formula_general_neu = arithmetic_intensity_triangle(xs_neu, fixed_T, fixed_T, fixed_r)
plt.plot(xs_neu, formula_general_neu, '-')
plt.xlabel('Neumann')
plt.ylabel(ylabel_ai)
plt.ylim(bottom=0, top=ylim_top_ai)
plt.savefig(dir_plot_prediction + 'formula_general_arithmetic_intensity_neumann.png', bbox_inches='tight')
plt.clf()

# Plot general arithmetic intensity halo
formula_general_halo = arithmetic_intensity_triangle(fixed_T, xs_halo, fixed_T, fixed_r)
plt.plot(xs_halo, formula_general_halo, '-')
plt.xlabel('Halo')
plt.ylabel(ylabel_ai)
plt.ylim(bottom=0, top=ylim_top_ai)
plt.savefig(dir_plot_prediction + 'formula_general_arithmetic_intensity_halo.png', bbox_inches='tight')
plt.clf()

# Plot general arithmetic intensity local
formula_general_T = arithmetic_intensity_triangle(fixed_T, fixed_T, xs_T, fixed_r)
plt.plot(xs_T, formula_general_T, '-')
plt.xlabel('Triangles')
plt.ylabel(ylabel_ai)
plt.ylim(bottom=0, top=ylim_top_ai)
plt.savefig(dir_plot_prediction + 'formula_general_arithmetic_intensity_triangles.png', bbox_inches='tight')
plt.clf()

# Plot general arithmetic intensity all
plt.plot(xs_neu, formula_general_neu, '-', label='Neumann')
plt.plot(xs_halo, formula_general_halo, '-', label='Halo')
plt.plot(xs_T, formula_general_T, '-', label='Triangles')
plt.legend()
plt.xlabel('Triangles')
plt.ylabel(ylabel_ai)
plt.ylim(bottom=0, top=ylim_top_ai)
plt.savefig(dir_plot_prediction + 'formula_general_arithmetic_intensity_all.png', bbox_inches='tight')
plt.clf()





############################
# Communication estimation #
############################

# 0: Ranks on same CPU
# 1: Ranks on different CPU
# 2: Ranks on different nodes

# [micro s]
alpha0 = 0.728319 / 1e6
alpha1 = 1.162463 / 1e6
alpha2 = 2.829573 / 1e6

# [GB/s]
beta0 = 7.218045 * 1e9
beta1 = 5.657298 * 1e9
beta2 = 2.013228 * 1e9

def t_comm(alpha, beta, t_halo):
	return ( 6 * ( (8*t_halo) / beta + alpha ) ) * 1e3

t_halo = np.linspace(350, 1500, 100)
t_comm_0 = t_comm(alpha0, beta0, t_halo)
t_comm_1 = t_comm(alpha1, beta1, t_halo)
t_comm_2 = t_comm(alpha2, beta2, t_halo)

plt.plot(t_halo, t_comm_2, '-', label='Different nodes')
plt.plot(t_halo, t_comm_1, '-', label='Different CPUs')
plt.plot(t_halo, t_comm_0, '-', label='Same CPU')
plt.legend()
plt.xlabel('Halo triangles')
plt.ylabel('Communication time [ms]')
plt.ylim(bottom=0)
plt.savefig(dir_plot_prediction + 'formula_hockney_communication', bbox_inches='tight')
plt.clf()
