import os
import numpy as np
from matplotlib import pyplot as plt

dir_out = 'out/'
dir_structure = dir_out + 'structure/'
dir_raw = dir_out + 'raw/'
dir_modified = dir_out + 'modified/'
dir_plot = dir_out + 'plot/'
dir_input = dir_out + 'input/'

structure_files = os.listdir(dir_structure)
raw_files = os.listdir(dir_raw)

structure_data = {}
raw_data = {}
raw_data_details = {}

# Constants
n_points_plot = 4
font_size = 15
ranks = 1
threads = 1
xlabel = "Number of parallel processes"
max_proc = 24

plt.rcParams.update({'font.size': font_size})

for structure_file in structure_files:
	with open(dir_structure + structure_file, 'r') as f:
		lines = [line.split() for line in f.readlines()]
		vertices_init = int(structure_file.split('v')[0])
		vertices = int(lines[0][0])
		elements = int(lines[0][1])
		max_upper = int(lines[1][0])
		max_lower = int(lines[1][1])
		n_local = int(lines[1][2])
		#n_neuman = int(lines[1][3])
		flop_tot = int(lines[2][0])
		mem_tot = int(lines[3][0])
		flopbyte = float(lines[4][0])
		p = int(lines[5][0])
		n_neuman = sum([int(line[4][:-1]) for line in lines[6:6+p]])
		structure_data[structure_file] = [vertices_init, vertices, elements, max_upper, max_lower, n_local, n_neuman, flop_tot, mem_tot, flopbyte, p]

for raw_file in raw_files:
	with open(dir_raw + raw_file, 'r') as f:
		lines = [line.split() for line in f.readlines()]
		nodes = int(raw_file.split('n')[0])
		p = int(lines[1][0])
		t = int(lines[2][0])
		vertices = int(lines[3][0])
		elements = int(lines[4][0])
		samples = int(lines[5][0])
		vertices_init = int(raw_file.split('-')[-1][:-5])
		structure_file = str(vertices_init) + 'v-' + str(p) + 'p.txt'
		raw_data[raw_file] = [structure_file, nodes, p, t, vertices_init, vertices, elements, samples]

		calc_avg, calc_avg_p1, calc_avg_p2, calc_avg_p3, halo_avg = [0.0]*p, [0.0]*p, [0.0]*p, [0.0]*p, [0.0]*p
		calc_var, calc_var_p1, calc_var_p2, calc_var_p3, halo_var = [0.0]*p, [0.0]*p, [0.0]*p, [0.0]*p, [0.0]*p
		for x in range(p):
			index = 7 + x * 7
			rank = int(lines[index][1])
			calc_avg[rank] = float(lines[index+1][0])
			calc_avg_p1[rank] = float(lines[index+2][0])
			calc_avg_p2[rank] = float(lines[index+3][0])
			calc_avg_p3[rank] = float(lines[index+4][0])
			halo_avg[rank] = float(lines[index+5][0])
			calc_var[rank] = float(lines[index+1][2])
			calc_var_p1[rank] = float(lines[index+2][2])
			calc_var_p2[rank] = float(lines[index+3][2])
			calc_var_p3[rank] = float(lines[index+4][2])
			halo_var[rank] = float(lines[index+5][2])
		raw_data_details[raw_file] = [calc_avg, calc_avg_p1, calc_avg_p2, calc_avg_p3, halo_avg]
		raw_data_details[raw_file] += [calc_var, calc_var_p1, calc_var_p2, calc_var_p3, halo_var]

		calc_avg_reduced = sum(calc_avg) / p
		calc_avg_p1_reduced = sum(calc_avg_p1) / p
		calc_avg_p2_reduced = sum(calc_avg_p2) / p
		calc_avg_p3_reduced = sum(calc_avg_p3) / p
		halo_avg_reduced = sum(halo_avg) / p
		raw_data[raw_file] += [calc_avg_reduced, calc_avg_p1_reduced, calc_avg_p2_reduced, calc_avg_p3_reduced, halo_avg_reduced]

# Data used for benchmarking
with open(dir_modified + "benchmark.txt", 'w') as f:
	for raw_file in raw_files:
		data = raw_data[raw_file]
		structure_file = data[0]
		nodes = data[1]
		p = data[2]
		t = data[3]
		vertices = data[5]
		elements = data[6]
		calc_avg_reduced = data[8]
		halo_avg_reduced = data[12]

		structure = structure_data[structure_file]
		max_upper = structure[3]
		max_lower = structure[4]
		n_local = structure[5]
		n_neuman = structure[6]
		flop_tot = structure[7]
		flopbyte = structure[9]
		flops = flop_tot / calc_avg_reduced

		print(raw_file, nodes, p, t, vertices, elements, max_upper, max_lower, n_local, n_neuman, calc_avg_reduced, 'ms', halo_avg_reduced, 'ms', flopbyte, flops, ('KFLOP/s'), file=f)


# Plot structure
structure_collection = {}
structure_borders = {}
for structure_file in structure_data:
	structure = structure_data[structure_file]
	vertices_init = structure[0]
	vertices = structure[1]
	elements = structure[2]
	max_upper = structure[3]
	max_lower = structure[4]
	max_border = max(max_upper, max_lower)
	n_local = structure[5]
	n_neuman = structure[6]
	p = structure[10]

	if vertices_init not in structure_collection:
		structure_collection[vertices_init] = {"V":[], "E":[], "p":[], "n_local":[], "max_upper":[], "max_lower":[], "max_border":[], "n_neuman":[]}

	if vertices_init not in structure_borders and p != 1:
		structure_borders[vertices_init] = []

	structure_coll = structure_collection[vertices_init]
	structure_coll["V"] += [vertices]
	structure_coll["E"] += [elements]
	structure_coll["p"] += [p]
	structure_coll["n_local"] += [n_local]
	structure_coll["max_upper"] += [max_upper]
	structure_coll["max_lower"] += [max_lower]
	structure_coll["max_border"] += [max_border]
	structure_coll["n_neuman"] += [n_neuman]

	if p != 1:
		structure_borders[vertices_init] += [max_border]
xs = sorted(list(structure_collection.keys()))
xs = [vertices_init for vertices_init in xs if vertices_init in structure_borders]
ys_elements = [structure_collection[vertices_init]["E"][0] for vertices_init in xs]
ys_neuman = [structure_collection[vertices_init]["n_neuman"][0] for vertices_init in xs]
ys_border = [sum(structure_borders[vertices_init])//len(structure_borders[vertices_init]) for vertices_init in xs]
if len(xs) > 3:
	xticks_v = [xs[0]] + xs[-3:]
	xs_np = np.linspace(min(xs), max(xs), 100)
else:
	xticks_v = xs
	xs_np = np.linspace(min(xs), max(xs), 100) if xs else np.linspace(0, 0, 100)
# Plot elements
elements_upper = 2 * xs_np - 4
elements_lower = xs_np - 2
plt.plot(xs_np, elements_upper, '--', label="2V-4")
plt.plot(xs_np, elements_lower, '--', label="V-2")
plt.plot(xs, ys_elements, '-o')
plt.xticks(xticks_v)
plt.legend()
plt.xlabel('Vertices')
plt.ylabel('Triangles')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'structure_elements.png', bbox_inches='tight')
plt.clf()
# Plot neuman
neuman_approx = 8.5 * np.sqrt(xs_np) * 1.12
#neuman_approx = 4 * np.sqrt(xs_np)
#neuman_upper = 2 * xs_np
#neuman_lower = 3 + xs_np * 0
plt.plot(xs_np, neuman_approx, '--', label="8.5 sqrt(V)*1.12")
#plt.plot(xs_np, neuman_approx, '--', label="4 sqrt(V)")
#plt.plot(xs_np, neuman_upper, '--', label="2V")
#plt.plot(xs_np, neuman_lower, '--', label="3")
plt.plot(xs, ys_neuman, '-o')
plt.xticks(xticks_v)
plt.legend()
plt.xlabel('Vertices')
plt.ylabel('Neumann triangles')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'structure_neuman.png', bbox_inches='tight')
plt.clf()
# Plot borders
border_approx = np.sqrt(xs_np) * 1.12
#border_approx = np.sqrt(xs_np)
#border_upper = xs_np / 2
#border_lower = xs_np * 0
plt.plot(xs_np, border_approx, '--', label="sqrt(V)*1.12")
#plt.plot(xs_np, border_upper, '--', label="V/2")
#plt.plot(xs_np, border_lower, '--', label="0")
plt.plot(xs, ys_border, '-o')
plt.xticks(xticks_v)
plt.legend()
plt.xlabel('Vertices')
plt.ylabel('Halo triangles')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'structure_border.png', bbox_inches='tight')
plt.clf()


# Plot communication
fixed_v = 200000
halo_times = {}
for raw_file in raw_files:
	data = raw_data[raw_file]
	p = data[2]
	t = data[3]
	vertices_init = data[4]
	vertices = data[5]
	elements = data[6]
	proc = p*t
	calc_avg_reduced = data[8]
	halo_time = data[12]
	calc_avg_percent = calc_avg_reduced / (calc_avg_reduced + halo_time)
	halo_time_percent = halo_time / (calc_avg_reduced + halo_time)

	if vertices_init not in halo_times:
		halo_times[vertices_init] = {"V":[], "E":[], "proc":[], "p":[], "t":[], "time":[], "halo_time":[], "time_percent":[], "halo_percent":[]}

	halo_data = halo_times[vertices_init]
	halo_data["V"] += [vertices]
	halo_data["E"] += [elements]
	halo_data["proc"] += [proc]
	halo_data["p"] += [p]
	halo_data["t"] += [t]
	halo_data["time"] += [calc_avg_reduced]
	halo_data["halo_time"] += [halo_time]
	halo_data["time_percent"] += [calc_avg_percent]
	halo_data["halo_percent"] += [halo_time_percent]
# Plot comp and comm
if len(halo_times) > 0:
	key_max = fixed_v if fixed_v in halo_times else max(halo_times)
	v_max = halo_times[key_max]
	order = np.argsort(v_max["p"])
	xs = [str(x) for x in np.array(v_max["p"])[order]]
	y_comm = np.array(v_max["halo_time"])[order]
	y_comp = np.array(v_max["time"])[order]
	plt.bar(xs, y_comm, label='Communication', bottom=y_comp)
	plt.bar(xs, y_comp, label='Computation')
	plt.legend()
	plt.xlabel('Ranks')
	plt.ylabel('Runtime [ms]')
	plt.ylim(bottom=0)
	plt.savefig(dir_plot + 'communication_overhead.png', bbox_inches='tight')
	plt.clf()
# Plot halo time
xticks_p = []
for k in sorted(halo_times, reverse=True):
	v = halo_times[k]
	if len(v["V"]) > n_points_plot:
		xticks_p += v["p"]
		order = np.argsort(v["p"])
		xs = np.array(v["p"])[order]
		ys = np.array(v["halo_time"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		plt.plot(xs_cut, ys_cut, '-o', label='V total: {}'.format(k))
xticks_p = sorted(set(xticks_p))
plt.xticks(xticks_p)
plt.legend()
plt.xlabel('Ranks')
plt.ylabel('Runtime [ms]')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'communication_time.png', bbox_inches='tight')
plt.clf()



# Plot p1, p2, p3
max_mem_s_K = -1
prediction_file = dir_input + "prediction_roof.txt"
if os.path.isfile(prediction_file):
	with open(prediction_file, 'r') as f:
		lines = [line.split() for line in f.readlines()]
		max_mem_s_K = float(lines[0][0]) * 1e6
fixed_t = 1
fixed_v = 200000
fixed_ranks = [1, 2, 3, 4, 6, 8, 12, 16, 24]
fixed_p = 4
runtime_parts = {}
runtime_parts_v = {}
for raw_file in raw_files:
	data = raw_data[raw_file]
	p = data[2]
	t = data[3]
	vertices_init = data[4]
	vertices = data[5]
	elements = data[6]
	proc = p*t
	calc_avg_reduced = data[8]
	calc_avg_p1_reduced = data[9]
	calc_avg_p2_reduced = data[10]
	calc_avg_p3_reduced = data[11]
	calc_avg_pn_sum = sum([calc_avg_p1_reduced, calc_avg_p2_reduced, calc_avg_p3_reduced])
	calc_p1_percent = calc_avg_p1_reduced / calc_avg_pn_sum
	calc_p2_percent = calc_avg_p2_reduced / calc_avg_pn_sum
	calc_p3_percent = calc_avg_p3_reduced / calc_avg_pn_sum


	if p in fixed_ranks:
		if vertices_init not in runtime_parts:
			runtime_parts[vertices_init] = {"V":[], "E":[], "proc":[], "p":[], "t":[], "time":[], "time_p1":[], "time_p2":[], "time_p3":[],
											"time_p1_percent":[], "time_p2_percent":[], "time_p3_percent":[]}

		if p not in runtime_parts_v:
			runtime_parts_v[p] = {"VINIT":[], "V":[], "E":[], "proc":[], "p":[], "t":[], "time":[], "time_p1":[], "time_p2":[], "time_p3":[],
									"time_p1_percent":[], "time_p2_percent":[], "time_p3_percent":[]}

		runtime_data = runtime_parts[vertices_init]
		runtime_data["V"] += [vertices]
		runtime_data["E"] += [elements]
		runtime_data["proc"] += [proc]
		runtime_data["p"] += [p]
		runtime_data["t"] += [t]
		runtime_data["time"] += [calc_avg_reduced]
		runtime_data["time_p1"] += [calc_avg_p1_reduced]
		runtime_data["time_p2"] += [calc_avg_p2_reduced]
		runtime_data["time_p3"] += [calc_avg_p3_reduced]
		runtime_data["time_p1_percent"] += [calc_p1_percent]
		runtime_data["time_p2_percent"] += [calc_p2_percent]
		runtime_data["time_p3_percent"] += [calc_p3_percent]

		runtime_data_v = runtime_parts_v[p]
		runtime_data_v["VINIT"] += [vertices_init]
		runtime_data_v["V"] += [vertices]
		runtime_data_v["E"] += [elements]
		runtime_data_v["proc"] += [proc]
		runtime_data_v["p"] += [p]
		runtime_data_v["t"] += [t]
		runtime_data_v["time"] += [calc_avg_reduced]
		runtime_data_v["time_p1"] += [calc_avg_p1_reduced]
		runtime_data_v["time_p2"] += [calc_avg_p2_reduced]
		runtime_data_v["time_p3"] += [calc_avg_p3_reduced]
		runtime_data_v["time_p1_percent"] += [calc_p1_percent]
		runtime_data_v["time_p2_percent"] += [calc_p2_percent]
		runtime_data_v["time_p3_percent"] += [calc_p3_percent]
# Plot p1, p2, p3 prediction from vertices_init for increasing ranks
if len(runtime_parts) > 0 and max_mem_s_K != -1:
	key_max = fixed_v if fixed_v in runtime_parts else max(runtime_parts)
	v_max = runtime_parts[key_max]
	order = np.argsort(v_max["p"])
	xs = [str(x) for x in np.array(v_max["p"])[order]]
	xs_np = [x for x in np.array(v_max["p"])[order]]
	p1 = [((8.5*np.sqrt(key_max)*1.12/r)*8 + (8.5*np.sqrt(key_max)*1.12/r + 2*np.sqrt(key_max)*1.12 + 2*key_max/r)*22) *8/(max_mem_s_K * max_proc/r) for r in xs_np]
	p2 = [((2*key_max/r)*47) *8/(max_mem_s_K * max_proc/r) for r in xs_np]
	p3 = [((2*key_max/r)*36) *8/(max_mem_s_K * max_proc/r) for r in xs_np]
	p_sum = [sum([p1[i],p2[i],p3[i]]) for i in range(len(xs_np))]
	#p1 = np.array([p1[i]/p_sum[i] for i in range(len(xs_np))])
	#p2 = np.array([p2[i]/p_sum[i] for i in range(len(xs_np))])
	#p3 = np.array([p3[i]/p_sum[i] for i in range(len(xs_np))])
	p1 = np.array(p1)
	p2 = np.array(p2)
	p3 = np.array(p3)
	p1 = np.flip(p1)
	p2 = np.flip(p2)
	p3 = np.flip(p3)
	xs = xs[::-1]
	xticks_p = [int(x) for x in xs]
	xticks_str = ['{}R,{}T'.format(p, max_proc//p) for p in xticks_p]
	plt.bar(xs, p1, label='P1', bottom=p2+p3)
	plt.bar(xs, p2, label='P2', bottom=p3)
	plt.bar(xs, p3, label='P3')
	plt.xticks(xs, xticks_str, rotation=90)
	plt.legend()
	#plt.xlabel("Ranks")
	#plt.ylabel('Predicted runtime proportion')
	plt.ylabel('Predicted runtime [ms]')
	plt.ylim(bottom=0)
	plt.savefig(dir_plot + 'parts_runtime_prediction_from_v.png', bbox_inches='tight')
	plt.clf()
# Plot p1, p2, p3 for increasing ranks
if len(runtime_parts) > 0:
	key_max = fixed_v if fixed_v in runtime_parts else max(runtime_parts)
	v_max = runtime_parts[key_max]
	order = np.argsort(v_max["p"])
	xs = [str(x) for x in np.array(v_max["p"])[order]]
	p1 = np.array(v_max["time_p1"])[order]
	p2 = np.array(v_max["time_p2"])[order]
	p3 = np.array(v_max["time_p3"])[order]
	p1 = np.flip(p1)
	p2 = np.flip(p2)
	p3 = np.flip(p3)
	xs = xs[::-1]
	xticks_p = [int(x) for x in xs]
	xticks_str = ['{}R,{}T'.format(p, max_proc//p) for p in xticks_p]
	plt.bar(xs, p1, label='P1', bottom=p2+p3)
	plt.bar(xs, p2, label='P2', bottom=p3)
	plt.bar(xs, p3, label='P3')
	plt.xticks(xs, xticks_str, rotation=90)
	plt.legend()
	#plt.xlabel("Ranks")
	#plt.ylabel('Runtime proportion')
	plt.ylabel('Runtime [ms]')
	plt.ylim(bottom=0)
	plt.savefig(dir_plot + 'parts_runtime.png', bbox_inches='tight')
	plt.clf()
# Plot p1, p2, p3 for increasing vertices
if len(runtime_parts_v) > 0:
	key_max = fixed_p if fixed_p in runtime_parts_v else max(runtime_parts_v)
	v_max = runtime_parts_v[key_max]
	order = np.argsort(v_max["VINIT"])
	xs = [str(x) for x in np.array(v_max["VINIT"])[order]]
	p1 = np.array(v_max["time_p1_percent"])[order]
	p2 = np.array(v_max["time_p2_percent"])[order]
	p3 = np.array(v_max["time_p3_percent"])[order]
	plt.bar(xs, p1, label='P1', bottom=p2+p3)
	plt.bar(xs, p2, label='P2', bottom=p3)
	plt.bar(xs, p3, label='P3')
	plt.legend()
	plt.xlabel("V")
	plt.ylabel('Runtime proportion')
	#plt.ylabel('Runtime [ms]')
	plt.ylim(bottom=0)
	plt.savefig(dir_plot + 'parts_runtime_vertices.png', bbox_inches='tight')
	plt.clf()


# Plot load balance
fixed_t = 1
fixed_v = 200000
fixed_ranks = [1, 2, 4, 8, 16, 24]
fixed_p = 4
balance_dist = {}
balance_dist_v = {}
for raw_file in raw_files:
	data = raw_data[raw_file]
	data_details = raw_data_details[raw_file]
	p = data[2]
	t = data[3]
	vertices_init = data[4]
	vertices = data[5]
	elements = data[6]
	proc = p*t
	calc_avg_reduced = data[8]
	calc_avgs = data_details[0]
	calc_avgs_percent = [avg/sum(calc_avgs) for avg in calc_avgs]

	if t == fixed_t and p in fixed_ranks:
		if vertices_init not in balance_dist:
			balance_dist[vertices_init] = {"V":[], "E":[], "proc":[], "p":[], "t":[], "time":[], "time_p":[], "time_p_percent":[]}

		if p not in balance_dist_v:
			balance_dist_v[p] = {"VINIT":[], "V":[], "E":[], "proc":[], "p":[], "t":[], "time":[], "time_p":[], "time_p_percent":[]}

		balance_data = balance_dist[vertices_init]
		balance_data["V"] += [vertices]
		balance_data["E"] += [elements]
		balance_data["proc"] += [proc]
		balance_data["p"] += [p]
		balance_data["t"] += [t]
		balance_data["time"] += [calc_avg_reduced]
		balance_data["time_p"] += [calc_avgs]
		balance_data["time_p_percent"] += [calc_avgs_percent]

		balance_data_v = balance_dist_v[p]
		balance_data_v["VINIT"] += [vertices_init]
		balance_data_v["V"] += [vertices]
		balance_data_v["E"] += [elements]
		balance_data_v["proc"] += [proc]
		balance_data_v["p"] += [p]
		balance_data_v["t"] += [t]
		balance_data_v["time"] += [calc_avg_reduced]
		balance_data_v["time_p"] += [calc_avgs]
		balance_data_v["time_p_percent"] += [calc_avgs_percent]
# Plot load balance for increasing ranks
if len(balance_dist) > 0:
	key_max = fixed_v if fixed_v in balance_dist else max(balance_dist)
	v_max = balance_dist[key_max]
	order = np.argsort(v_max["p"])
	xs = [str(x) for x in np.array(v_max["p"])[order]]
	bars = [[0.0]*len(xs) for _ in range(max(v_max["p"]))]
	time_avgs_p = np.array(v_max["time_p_percent"])[order]
	for i in range(len(time_avgs_p)):
		time_avgs = time_avgs_p[i]
		for j in range(len(time_avgs)):
			bars[j][i] = time_avgs[j]
	bars = [np.array(bar) for bar in bars]
	bars_bottom = [sum(bars[:i]) for i in range(len(bars))]
	[plt.bar(xs, bars[i], bottom=bars_bottom[i]) for i in range(len(bars))]
	plt.xlabel("Ranks")
	plt.ylabel('Runtime proportion')
	plt.ylim(bottom=0)
	plt.savefig(dir_plot + 'load_balance.png', bbox_inches='tight')
	plt.clf()
# Plot load balance for increasing vertices
if len(balance_dist_v) > 0:
	key_max = fixed_p if fixed_p in balance_dist_v else max(balance_dist_v)
	v_max = balance_dist_v[key_max]
	order = np.argsort(v_max["VINIT"])
	xs = [str(x) for x in np.array(v_max["VINIT"])[order]]
	bars = [[0.0]*len(xs) for _ in range(max(v_max["p"]))]
	time_avgs_p = np.array(v_max["time_p_percent"])[order]
	for i in range(len(time_avgs_p)):
		time_avgs = time_avgs_p[i]
		for j in range(len(time_avgs)):
			bars[j][i] = time_avgs[j]
	bars = [np.array(bar) for bar in bars]
	bars_bottom = [sum(bars[:i]) for i in range(len(bars))]
	[plt.bar(xs, bars[i], bottom=bars_bottom[i]) for i in range(len(bars))]
	plt.xlabel("V")
	plt.ylabel('Runtime proportion')
	plt.ylim(bottom=0)
	plt.savefig(dir_plot + 'load_balance_vertices.png', bbox_inches='tight')
	plt.clf()



# Plot prediction
max_mem_s_K = -1
prediction_file = dir_input + "prediction_roof.txt"
if os.path.isfile(prediction_file):
	with open(prediction_file, 'r') as f:
		lines = [line.split() for line in f.readlines()]
		max_mem_s_K = float(lines[0][0]) * 1e6
strong_prediction = {}
weak_prediction = {}
for raw_file in raw_files:
	data = raw_data[raw_file]
	structure_file = data[0]
	p = data[2]
	t = data[3]
	vertices_init = data[4]
	vertices = data[5]
	elements = data[6]
	proc = p*t
	vertices_per_proc = vertices_init // proc
	mem_tot = structure_data[structure_file][8]

	if max_mem_s_K == -1:
		time_prediction = mem_tot / 1e9 / t
	else:
		time_prediction = mem_tot / max_mem_s_K / t

	if vertices_init not in strong_prediction:
		strong_prediction[vertices_init] = {"V":[], "E":[], "proc":[], "p":[], "t":[], "time_prediction":[]}

	if vertices_per_proc not in weak_prediction:
		weak_prediction[vertices_per_proc] = {"VINIT":[], "V":[], "E":[], "proc":[], "p":[], "t":[], "time_prediction":[]}

	strong_data = strong_prediction[vertices_init]
	strong_data["V"] += [vertices]
	strong_data["E"] += [elements]
	strong_data["proc"] += [proc]
	strong_data["p"] += [p]
	strong_data["t"] += [t]
	strong_data["time_prediction"] += [time_prediction]

	weak_data = weak_prediction[vertices_per_proc]
	weak_data["VINIT"] += [vertices_init]
	weak_data["V"] += [vertices]
	weak_data["E"] += [elements]
	weak_data["proc"] += [proc]
	weak_data["p"] += [p]
	weak_data["t"] += [t]
	weak_data["time_prediction"] += [time_prediction]
# Plot strong prediction from only vertices_init
strong_xticks_proc = []
for k in sorted(strong_prediction, reverse=True):
	v = strong_prediction[k]
	if len(set(v["proc"])) > n_points_plot and 1 in v["proc"]:
		if len(set(v["t"])) == 1:
			fixed_threads = v["t"][0]
			strong_xticks_proc += v["p"]
			order = np.argsort(v["p"])
			xs = np.array(v["p"])[order]
			xs_np = np.linspace(min(xs), max(xs), 100)
			if max_mem_s_K == -1:
				ys = ( ((8.5*np.sqrt(k)*1.12/xs_np) * 8 + (8.5*np.sqrt(k)*1.12/xs_np + 2*np.sqrt(k)*1.12 + 2*k/xs_np) * 22) +\
				((2*k/xs_np) * 47) + ((2*k/xs_np) * 36) )\
				* 8 / 1e9 / fixed_threads
			else:
				ys = ( ((8.5*np.sqrt(k)*1.12/xs_np) * 8 + (8.5*np.sqrt(k)*1.12/xs_np + 2*np.sqrt(k)*1.12 + 2*k/xs_np) * 22) +\
				((2*k/xs_np) * 47) + ((2*k/xs_np) * 36) )\
				* 8 / max_mem_s_K / fixed_threads
			plt.plot(xs_np, ys, '-', label='V total: {}'.format(k))
		elif len(set(v["p"])) == 1:
			fixed_ranks = v["p"][0]
			strong_xticks_proc += v["t"]
			order = np.argsort(v["t"])
			xs = np.array(v["t"])[order]
			xs_np = np.linspace(min(xs), max(xs), 100)
			if max_mem_s_K == -1:
				ys = ( ((8.5*np.sqrt(k)*1.12/fixed_ranks) * 8 + (8.5*np.sqrt(k)*1.12/fixed_ranks + 2*np.sqrt(k)*1.12 + 2*k/fixed_ranks) * 22) +\
				((2*k/fixed_ranks) * 47) + ((2*k/fixed_ranks) * 36) )\
				* 8 / 1e9 / xs_np
			else:
				ys = ( ((8.5*np.sqrt(k)*1.12/fixed_ranks) * 8 + (8.5*np.sqrt(k)*1.12/fixed_ranks + 2*np.sqrt(k)*1.12 + 2*k/fixed_ranks) * 22) +\
				((2*k/fixed_ranks) * 47) + ((2*k/fixed_ranks) * 36) )\
				* 8 / max_mem_s_K / xs_np
			plt.plot(xs_np, ys, '-', label='V total: {}'.format(k))
strong_xticks_proc = sorted(set(strong_xticks_proc))
strong_xticks_str = ["{}R,{}T".format(ranks, x) for x in strong_xticks_proc]
plt.xticks(strong_xticks_proc)
#plt.xticks(strong_xticks_proc, strong_xticks_str, rotation=90)
if max_mem_s_K == -1:
	ylabel = 'Memory operations [GB]'
else:
	ylabel = 'Predicted runtime [ms]'
plt.legend()
plt.xlabel(xlabel)
plt.ylabel(ylabel)
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'time_strong_predicion_from_v.png', bbox_inches='tight')
plt.clf()
# Plot strong prediction
strong_xticks_proc = []
for k in sorted(strong_prediction, reverse=True):
	v = strong_prediction[k]
	if len(set(v["proc"])) > n_points_plot and 1 in v["proc"]:
		strong_xticks_proc += v["proc"]
		order = np.argsort(v["proc"])
		xs = np.array(v["proc"])[order]
		ys = np.array(v["time_prediction"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		plt.plot(xs_cut, ys_cut, '-o', label='V total: {}'.format(k))
strong_xticks_proc = sorted(set(strong_xticks_proc))
strong_xticks_str = ["{}R,{}T".format(ranks, x) for x in strong_xticks_proc]
plt.xticks(strong_xticks_proc)
#plt.xticks(strong_xticks_proc, strong_xticks_str, rotation=90)
if max_mem_s_K == -1:
	ylabel = 'Memory operations [GB]'
else:
	ylabel = 'Predicted runtime [ms]'
plt.legend()
plt.xlabel(xlabel)
plt.ylabel(ylabel)
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'time_strong_predicion.png', bbox_inches='tight')
plt.clf()
# Plot weak prediction from only vertices_init
weak_xticks_proc = []
for k in sorted(weak_prediction, reverse=True):
	v = weak_prediction[k]
	if len(set(v["proc"])) > n_points_plot and 1 in v["proc"]:
		if len(set(v["t"])) == 1:
			fixed_threads = v["t"][0]
			k_weak = k * fixed_threads
			weak_xticks_proc += v["p"]
			order = np.argsort(v["p"])
			xs = np.array(v["p"])[order]
			xs_np = np.linspace(min(xs), max(xs), 100)
			if max_mem_s_K == -1:
				ys = ( ((8.5*np.sqrt(k_weak*xs_np)*1.12/xs_np) * 8 + (8.5*np.sqrt(k_weak*xs_np)*1.12/xs_np + 2*np.sqrt(k_weak*xs_np)*1.12 + 2*k_weak) * 22) +\
				((2*k_weak) * 47) + ((2*k_weak) * 36) )\
				* 8 / 1e9 / fixed_threads
			else:
				ys = ( ((8.5*np.sqrt(k_weak*xs_np)*1.12/xs_np) * 8 + (8.5*np.sqrt(k_weak*xs_np)*1.12/xs_np + 2*np.sqrt(k_weak*xs_np)*1.12 + 2*k_weak) * 22) +\
				((2*k_weak) * 47) + ((2*k_weak) * 36) )\
				* 8 / max_mem_s_K / fixed_threads
			plt.plot(xs_np, ys, '-', label='V per proc: {}'.format(k))
		elif len(set(v["p"])) == 1:
			fixed_ranks = v["p"][0]
			k_weak = k * fixed_ranks
			weak_xticks_proc += v["t"]
			order = np.argsort(v["t"])
			xs = np.array(v["t"])[order]
			xs_np = np.linspace(min(xs), max(xs), 100)
			if max_mem_s_K == -1:
				ys = ( ((8.5*np.sqrt(k_weak*xs_np)*1.12/fixed_ranks) * 8 + (8.5*np.sqrt(k_weak*xs_np)*1.12/fixed_ranks + 2*np.sqrt(k_weak*xs_np)*1.12 + 2*k*xs_np) * 22) +\
				((2*k*xs_np) * 47) + ((2*k*xs_np) * 36) )\
				* 8 / 1e9 / xs_np
			else:
				ys = ( ((8.5*np.sqrt(k_weak*xs_np)*1.12/fixed_ranks) * 8 + (8.5*np.sqrt(k_weak*xs_np)*1.12/fixed_ranks + 2*np.sqrt(k_weak*xs_np)*1.12 + 2*k*xs_np) * 22) +\
				((2*k*xs_np) * 47) + ((2*k*xs_np) * 36) )\
				* 8 / max_mem_s_K / xs_np
			plt.plot(xs_np, ys, '-', label='V per proc: {}'.format(k))
weak_xticks_proc = sorted(set(weak_xticks_proc))
weak_xticks_str = ["{}R,{}T".format(ranks, x) for x in weak_xticks_proc]
plt.xticks(weak_xticks_proc)
#plt.xticks(weak_xticks_proc, weak_xticks_str, rotation=90)
if max_mem_s_K == -1:
	ylabel = 'Memory operations [GB]'
else:
	ylabel = 'Predicted runtime [ms]'
plt.legend(bbox_to_anchor=[1.0,0.95])
plt.xlabel(xlabel)
plt.ylabel(ylabel)
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'time_weak_prediction_from_v.png', bbox_inches='tight')
plt.clf()
# Plot weak prediction
weak_xticks_proc = []
for k in sorted(weak_prediction, reverse=True):
	v = weak_prediction[k]
	if len(set(v["proc"])) > n_points_plot and 1 in v["proc"]:
		weak_xticks_proc += v["proc"]
		order = np.argsort(v["proc"])
		xs = np.array(v["proc"])[order]
		ys = np.array(v["time_prediction"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		plt.plot(xs_cut, ys_cut, '-o', label='V per proc: {}'.format(k))
weak_xticks_proc = sorted(set(weak_xticks_proc))
weak_xticks_str = ["{}R,{}T".format(ranks, x) for x in weak_xticks_proc]
plt.xticks(weak_xticks_proc)
#plt.xticks(weak_xticks_proc, weak_xticks_str, rotation=90)
if max_mem_s_K == -1:
	ylabel = 'Memory operations [GB]'
else:
	ylabel = 'Predicted runtime [ms]'
plt.legend(bbox_to_anchor=[1.0,0.95])
plt.xlabel(xlabel)
plt.ylabel(ylabel)
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'time_weak_prediction.png', bbox_inches='tight')
plt.clf()


# Plot time, speedup and efficiency graf
strong_scale = {}
strong_serial = {}
weak_scale = {}
weak_serial = {}
for raw_file in raw_files:
	data = raw_data[raw_file]
	p = data[2]
	t = data[3]
	vertices_init = data[4]
	vertices = data[5]
	elements = data[6]
	calc_avg_reduced = data[8]
	proc = p*t
	vertices_per_proc = vertices_init // proc

	if vertices_init not in strong_scale:
		strong_scale[vertices_init] = {"V":[], "E":[], "proc":[], "p":[], "t":[], "time":[], "speedup":[], "efficiency":[]}

	if vertices_per_proc not in weak_scale:
		weak_scale[vertices_per_proc] = {"VINIT":[], "V":[], "E":[], "proc":[], "p":[], "t":[], "time":[], "speedup":[], "efficiency":[]}

	if proc == 1:
		strong_serial[vertices_init] = {"proc":proc, "time": calc_avg_reduced}
		weak_serial[vertices_per_proc] = {"proc":proc, "time": calc_avg_reduced}

	strong_data = strong_scale[vertices_init]
	strong_data["V"] += [vertices]
	strong_data["E"] += [elements]
	strong_data["proc"] += [proc]
	strong_data["p"] += [p]
	strong_data["t"] += [t]
	strong_data["time"] += [calc_avg_reduced]

	weak_data = weak_scale[vertices_per_proc]
	weak_data["VINIT"] += [vertices_init]
	weak_data["V"] += [vertices]
	weak_data["E"] += [elements]
	weak_data["proc"] += [proc]
	weak_data["p"] += [p]
	weak_data["t"] += [t]
	weak_data["time"] += [calc_avg_reduced]
for k, v in strong_scale.items():
	if k in strong_serial:
		time_serial = strong_serial[k]["time"]
		for i in range(len(v["V"])):
			speedup = time_serial / v["time"][i]
			efficiency = speedup / v["proc"][i]
			v["speedup"] += [speedup]
			v["efficiency"] += [efficiency]
for k, v in weak_scale.items():
	if k in weak_serial:
		time_serial = weak_serial[k]["time"]
		for i in range(len(v["V"])):
			speedup = v["proc"][i] * time_serial / v["time"][i]
			efficiency = speedup / v["proc"][i]
			v["speedup"] += [speedup]
			v["efficiency"] += [efficiency]
# Plot time per proc
strong_xticks_proc = []
for k in sorted(strong_serial, reverse=True):
	v = strong_scale[k]
	if len(v["V"]) > n_points_plot:
		strong_xticks_proc += v["proc"]
		order = np.argsort(v["proc"])
		xs = np.array(v["proc"])[order]
		ys = np.array(v["time"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		plt.plot(xs_cut, ys_cut, '-o', label='V total: {}'.format(k))
strong_xticks_proc = sorted(set(strong_xticks_proc))
strong_xticks_str = ["{}R,{}T".format(ranks, x) for x in strong_xticks_proc]
plt.xticks(strong_xticks_proc)
#plt.xticks(strong_xticks_proc, strong_xticks_str, rotation=90)
plt.legend()
plt.xlabel(xlabel)
plt.ylabel('Runtime [ms]')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'time_strong.png', bbox_inches='tight')
plt.clf()
weak_xticks_proc = []
for k in sorted(weak_serial, reverse=True):
	v = weak_scale[k]
	if len(v["V"]) > n_points_plot:
		weak_xticks_proc += v["proc"]
		order = np.argsort(v["proc"])
		xs = np.array(v["proc"])[order]
		ys = np.array(v["time"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		plt.plot(xs_cut, ys_cut, '-o', label='V per proc: {}'.format(k))
weak_xticks_proc = sorted(set(weak_xticks_proc))
weak_xticks_str = ["{}R,{}T".format(ranks, x) for x in weak_xticks_proc]
plt.xticks(weak_xticks_proc)
#plt.xticks(weak_xticks_proc, weak_xticks_str, rotation=90)
plt.legend()
plt.xlabel(xlabel)
plt.ylabel('Runtime [ms]')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'time_weak.png', bbox_inches='tight')
plt.clf()
# Plot speedup
for k in sorted(strong_serial, reverse=True):
	v = strong_scale[k]
	if len(v["V"]) > n_points_plot:
		order = np.argsort(v["proc"])
		xs = np.array(v["proc"])[order]
		ys = np.array(v["speedup"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		plt.plot(xs_cut, ys_cut, '-o', label='V total: {}'.format(k))
plt.xticks(strong_xticks_proc)
#plt.xticks(strong_xticks_proc, strong_xticks_str, rotation=90)
plt.legend()
plt.xlabel(xlabel)
plt.ylabel('Speedup')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'speedup_strong.png', bbox_inches='tight')
plt.clf()
for k in sorted(weak_serial, reverse=True):
	v = weak_scale[k]
	if len(v["V"]) > n_points_plot:
		order = np.argsort(v["proc"])
		xs = np.array(v["proc"])[order]
		ys = np.array(v["speedup"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		plt.plot(xs_cut, ys_cut, '-o', label='V per proc: {}'.format(k))
plt.xticks(weak_xticks_proc)
#plt.xticks(weak_xticks_proc, weak_xticks_str, rotation=90)
plt.legend()
plt.xlabel(xlabel)
plt.ylabel('Speedup')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'speedup_weak.png', bbox_inches='tight')
plt.clf()
# Plot efficiency
for k in sorted(strong_serial, reverse=True):
	v = strong_scale[k]
	if len(v["V"]) > n_points_plot:
		order = np.argsort(v["proc"])
		xs = np.array(v["proc"])[order]
		ys = np.array(v["efficiency"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		plt.plot(xs_cut, ys_cut, '-o', label='V total: {}'.format(k))
plt.xticks(strong_xticks_proc)
#plt.xticks(strong_xticks_proc, strong_xticks_str, rotation=90)
plt.legend()
plt.xlabel(xlabel)
plt.ylabel('Efficiency')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'efficiency_strong.png', bbox_inches='tight')
plt.clf()
for k in sorted(weak_serial, reverse=True):
	v = weak_scale[k]
	if len(v["V"]) > n_points_plot:
		order = np.argsort(v["proc"])
		xs = np.array(v["proc"])[order]
		ys = np.array(v["efficiency"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		plt.plot(xs_cut, ys_cut, '-o', label='V per proc: {}'.format(k))
plt.xticks(weak_xticks_proc)
#plt.xticks(weak_xticks_proc, weak_xticks_str, rotation=90)
plt.legend()
plt.xlabel(xlabel)
plt.ylabel('Efficiency')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'efficiency_weak.png', bbox_inches='tight')
plt.clf()
[print(k,strong_scale[k]) for k in strong_scale]
[print('-',k,strong_serial[k]) for k in strong_serial]
print()
[print(k,weak_scale[k]) for k in weak_scale]
[print('-',k,weak_serial[k]) for k in weak_serial]




# Plot prediction of full utilization of p and t
max_mem_s_K = -1
prediction_file = dir_input + "prediction_roof.txt"
if os.path.isfile(prediction_file):
	with open(prediction_file, 'r') as f:
		lines = [line.split() for line in f.readlines()]
		max_mem_s_K = float(lines[0][0]) * 1e6
max_proc_scale = {}
for k, v in structure_data.items():
	vertices_init = v[0]
	vertices = v[1]
	elements = v[2]
	max_upper = v[3]
	max_lower = v[4]
	n_local = v[5]
	n_neuman = v[6]
	mem_tot = v[8]
	p = v[10]
	t = max_proc // p
	proc = p * t

	if p > max_proc:
		continue

	if max_mem_s_K == -1:
		time_prediction = mem_tot / 1e9 / t
	else:
		time_prediction = mem_tot / max_mem_s_K / t

	if proc == max_proc:
		if vertices_init not in max_proc_scale:
			max_proc_scale[vertices_init] = {"V":[], "E":[], "max_upper":[], "max_lower":[], "n_local":[], "n_neuman":[], "p":[], "t":[], "time_prediction":[]}

		max_proc_data = max_proc_scale[vertices_init]
		max_proc_data["V"] += [vertices]
		max_proc_data["E"] += [elements]
		max_proc_data["max_upper"] += [max_upper]
		max_proc_data["max_lower"] += [max_lower]
		max_proc_data["n_local"] += [n_local]
		max_proc_data["n_neuman"] += [n_neuman]
		max_proc_data["p"] += [p]
		max_proc_data["t"] += [t]
		max_proc_data["time_prediction"] += [time_prediction]
# Plot time for p and t from only vertices_init
xticks_t = []
for k in sorted(max_proc_scale, reverse=True):
	v = max_proc_scale[k]
	if len(v["V"]) > 1:
		xticks_t += v["t"]
		order = np.argsort(v["t"])
		xs = np.array(v["t"])[order]
		xs_np = np.linspace(min(xs), max(xs), 100)
		if max_mem_s_K == -1:
			ys = ( ((8.5*np.sqrt(k)*1.12/(max_proc/xs_np)) * 8 + (8.5*np.sqrt(k)*1.12/(max_proc/xs_np) + 2*np.sqrt(k)*1.12 + 2*k/(max_proc/xs_np)) * 22) +\
			((2*k/(max_proc/xs_np)) * 47) + ((2*k/(max_proc/xs_np)) * 36) )\
			* 8 / 1e9 / xs_np
		else:
			ys = ( ((8.5*np.sqrt(k)*1.12/(max_proc/xs_np)) * 8 + (8.5*np.sqrt(k)*1.12/(max_proc/xs_np) + 2*np.sqrt(k)*1.12 + 2*k/(max_proc/xs_np)) * 22) +\
			((2*k/(max_proc/xs_np)) * 47) + ((2*k/(max_proc/xs_np)) * 36) )\
			* 8 / max_mem_s_K / xs_np
		plt.plot(xs_np, ys, '-', label='V total: {}'.format(k))
xticks_t = sorted(set(xticks_t))
xticks_str = ['{}R,{}T'.format(max_proc//t, t) for t in xticks_t]
plt.xticks(xticks_t, xticks_str, rotation=90)
plt.legend(bbox_to_anchor=[1.0,0.95])
if max_mem_s_K == -1:
	ylabel = 'Memory operations [GB]'
else:
	ylabel = 'Predicted runtime [ms]'
plt.ylabel(ylabel)
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'max_proc_time_prediction_from_v.png', bbox_inches='tight')
plt.clf()
# Plot time for p and t
xticks_t = []
for k in sorted(max_proc_scale, reverse=True):
	v = max_proc_scale[k]
	if len(v["V"]) > 1:
		xticks_t += v["t"]
		order = np.argsort(v["t"])
		xs = np.array(v["t"])[order]
		ys = np.array(v["time_prediction"])[order]
		plt.plot(xs, ys, '-o', label='V total: {}'.format(k))
xticks_t = sorted(set(xticks_t))
xticks_str = ['{}R,{}T'.format(max_proc//t, t) for t in xticks_t]
plt.xticks(xticks_t, xticks_str, rotation=90)
plt.legend(bbox_to_anchor=[1.0,0.95])
if max_mem_s_K == -1:
	ylabel = 'Memory operations [GB]'
else:
	ylabel = 'Predicted runtime [ms]'
plt.ylabel(ylabel)
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'max_proc_time_prediction.png', bbox_inches='tight')
plt.clf()



# Plot full utilization of p and t
max_proc_scale = {}
max_proc_serial = {}
for raw_file in raw_files:
	data = raw_data[raw_file]
	p = data[2]
	t = data[3]
	vertices_init = data[4]
	vertices = data[5]
	elements = data[6]
	calc_avg_reduced = data[8]
	proc = p*t

	if proc == 1:
		max_proc_serial[vertices_init] = {"proc":proc, "time": calc_avg_reduced}

	if proc == max_proc:
		if vertices_init not in max_proc_scale:
			max_proc_scale[vertices_init] = {"V":[], "E":[], "proc":[], "p":[], "t":[], "time":[], "speedup":[], "efficiency":[]}

		max_proc_data = max_proc_scale[vertices_init]
		max_proc_data["V"] += [vertices]
		max_proc_data["E"] += [elements]
		max_proc_data["proc"] += [proc]
		max_proc_data["p"] += [p]
		max_proc_data["t"] += [t]
		max_proc_data["time"] += [calc_avg_reduced]
max_proc_serial = {k:v for k,v in max_proc_serial.items() if k in max_proc_scale}
for k, v in max_proc_scale.items():
	if k in max_proc_serial:
		time_serial = max_proc_serial[k]["time"]
		for i in range(len(v["V"])):
			speedup = time_serial / v["time"][i]
			efficiency = speedup / v["proc"][i]
			v["speedup"] += [speedup]
			v["efficiency"] += [efficiency]
# Plot time for p and t
xticks_t = []
for k in sorted(max_proc_scale, reverse=True):
	v = max_proc_scale[k]
	if len(v["V"]) > 1:
		xticks_t += v["t"]
		order = np.argsort(v["t"])
		xs = np.array(v["t"])[order]
		ys = np.array(v["time"])[order]
		plt.plot(xs, ys, '-o', label='V total: {}'.format(k))
xticks_t = sorted(set(xticks_t))
xticks_str = ['{}R,{}T'.format(max_proc//t, t) for t in xticks_t]
plt.xticks(xticks_t, xticks_str, rotation=90)
plt.legend()
plt.ylabel('Runtime [ms]')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'max_proc_time.png', bbox_inches='tight')
plt.clf()
# Plot speedup for p and t
for k in sorted(max_proc_serial, reverse=True):
	v = max_proc_scale[k]
	if len(v["V"]) > 1:
		order = np.argsort(v["p"])
		xs = np.array(v["p"])[order]
		ys = np.array(v["speedup"])[order]
		plt.plot(xs, ys, '-o', label='V total: {}'.format(k))
plt.xticks(xticks_t, xticks_str, rotation=90)
plt.legend()
plt.ylabel('Speedup')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'max_proc_speedup.png', bbox_inches='tight')
plt.clf()
# Plot efficiency for p and t
for k in sorted(max_proc_serial, reverse=True):
	v = max_proc_scale[k]
	if len(v["V"]) > 1:
		order = np.argsort(v["p"])
		xs = np.array(v["p"])[order]
		ys = np.array(v["efficiency"])[order]
		plt.plot(xs, ys, '-o', label='V total: {}'.format(k))
plt.xticks(xticks_t, xticks_str, rotation=90)
plt.legend()
plt.ylabel('Efficiency')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'max_proc_efficiency.png', bbox_inches='tight')
plt.clf()
