import os
import numpy as np
from matplotlib import pyplot as plt

dir_out = 'out/'
dir_structure = dir_out + 'structure/'
dir_raw = dir_out + 'raw/'
dir_modified = dir_out + 'modified/'
dir_plot = dir_out + 'plot/'
dir_input = dir_out + 'input/'

structure_files = os.listdir(dir_structure)
raw_files = os.listdir(dir_raw)

structure_data = {}
raw_data = {}
raw_data_details = {}

# Constants
n_points_plot = 2
#labels_limit_t = [6, 4, 3]
labels_limit_t = [6, 4]
font_size = 15
ranks = 1
threads = 1
xlabel = "Nodes"
max_proc = 24

plt.rcParams.update({'font.size': font_size})

for structure_file in structure_files:
	with open(dir_structure + structure_file, 'r') as f:
		lines = [line.split() for line in f.readlines()[:6]]
		vertices_init = int(structure_file.split('v')[0])
		vertices = int(lines[0][0])
		elements = int(lines[0][1])
		max_upper = int(lines[1][0])
		max_lower = int(lines[1][1])
		n_local = int(lines[1][2])
		n_neuman = int(lines[1][3])
		flop_tot = int(lines[2][0])
		mem_tot = int(lines[3][0])
		flopbyte = float(lines[4][0])
		p = int(lines[5][0])
		structure_data[structure_file] = [vertices_init, vertices, elements, max_upper, max_lower, n_local, n_neuman, flop_tot, mem_tot, flopbyte, p]

for raw_file in raw_files:
	with open(dir_raw + raw_file, 'r') as f:
		lines = [line.split() for line in f.readlines()]
		nodes = int(raw_file.split('n')[0])
		p = int(lines[1][0])
		t = int(lines[2][0])
		vertices = int(lines[3][0])
		elements = int(lines[4][0])
		samples = int(lines[5][0])
		vertices_init = int(raw_file.split('-')[-1][:-5])
		structure_file = str(vertices_init) + 'v-' + str(p) + 'p.txt'
		raw_data[raw_file] = [structure_file, nodes, p, t, vertices_init, vertices, elements, samples]

		calc_avg, calc_avg_p1, calc_avg_p2, calc_avg_p3, halo_avg = [0.0]*p, [0.0]*p, [0.0]*p, [0.0]*p, [0.0]*p
		calc_var, calc_var_p1, calc_var_p2, calc_var_p3, halo_var = [0.0]*p, [0.0]*p, [0.0]*p, [0.0]*p, [0.0]*p
		for x in range(p):
			index = 7 + x * 7
			rank = int(lines[index][1])
			calc_avg[rank] = float(lines[index+1][0])
			calc_avg_p1[rank] = float(lines[index+2][0])
			calc_avg_p2[rank] = float(lines[index+3][0])
			calc_avg_p3[rank] = float(lines[index+4][0])
			halo_avg[rank] = float(lines[index+5][0])
			calc_var[rank] = float(lines[index+1][2])
			calc_var_p1[rank] = float(lines[index+2][2])
			calc_var_p2[rank] = float(lines[index+3][2])
			calc_var_p3[rank] = float(lines[index+4][2])
			halo_var[rank] = float(lines[index+5][2])
		raw_data_details[raw_file] = [calc_avg, calc_avg_p1, calc_avg_p2, calc_avg_p3, halo_avg]
		raw_data_details[raw_file] += [calc_var, calc_var_p1, calc_var_p2, calc_var_p3, halo_var]

		calc_avg_reduced = sum(calc_avg) / p
		calc_avg_p1_reduced = sum(calc_avg_p1) / p
		calc_avg_p2_reduced = sum(calc_avg_p2) / p
		calc_avg_p3_reduced = sum(calc_avg_p3) / p
		halo_avg_reduced = sum(halo_avg) / p
		raw_data[raw_file] += [calc_avg_reduced, calc_avg_p1_reduced, calc_avg_p2_reduced, calc_avg_p3_reduced, halo_avg_reduced]

# Data used for benchmarking
with open(dir_modified + "benchmark.txt", 'w') as f:
	for raw_file in raw_files:
		data = raw_data[raw_file]
		structure_file = data[0]
		nodes = data[1]
		p = data[2]
		t = data[3]
		vertices = data[5]
		elements = data[6]
		calc_avg_reduced = data[8]
		halo_avg_reduced = data[12]

		structure = structure_data[structure_file]
		max_upper = structure[3]
		max_lower = structure[4]
		n_local = structure[5]
		n_neuman = structure[6]
		flop_tot = structure[7]
		flopbyte = structure[9]
		flops = flop_tot / calc_avg_reduced

		print(raw_file, nodes, p, t, vertices, elements, max_upper, max_lower, n_local, n_neuman, calc_avg_reduced, 'ms', halo_avg_reduced, 'ms', flopbyte, flops, ('KFLOP/s'), file=f)



# Plot communication
fixed_v = 200000
fixed_t = 6
fixed_key = (fixed_v, fixed_t)
n_halo_points = 0
halo_core_limit = 0.1
halo_node_limit0 = 0.4
halo_node_limit1 = 0.8
halo_times = {}
halo_times_weak = {}
for raw_file in raw_files:
	data = raw_data[raw_file]
	n = data[1]
	p = data[2]
	t = data[3]
	vertices_init = data[4]
	vertices = data[5]
	elements = data[6]
	proc = p*t
	vertices_per_n = vertices_init // n
	calc_avg_reduced = data[8]
	halo_time = data[12]
	calc_avg_percent = calc_avg_reduced / (calc_avg_reduced + halo_time)
	halo_time_percent = halo_time / (calc_avg_reduced + halo_time)
	halo_time_details = raw_data_details[raw_file][4]

	structure_file = data[0]
	max_upper = structure_data[structure_file][3]
	max_lower = structure_data[structure_file][4]
	max_border = max(max_upper, max_lower)

	key_strong = (vertices_init, t)
	key_weak = (vertices_per_n, t)

	if key_strong not in halo_times:
		halo_times[key_strong] = {"V":[], "E":[], "border":[], "proc":[], "n":[], "p":[], "t":[], "time":[], "halo_time":[], "time_percent":[], "halo_percent":[], "halo_details":[]}

	if key_weak not in halo_times_weak:
		halo_times_weak[key_weak] = {"VINIT":[], "V":[], "E":[], "border":[], "proc":[], "n":[], "p":[], "t":[], "time":[], "halo_time":[], "time_percent":[], "halo_percent":[], "halo_details":[]}

	halo_data = halo_times[key_strong]
	halo_data["V"] += [vertices]
	halo_data["E"] += [elements]
	halo_data["border"] += [max_border]
	halo_data["proc"] += [proc]
	halo_data["n"] += [n]
	halo_data["p"] += [p]
	halo_data["t"] += [t]
	halo_data["time"] += [calc_avg_reduced]
	halo_data["halo_time"] += [halo_time]
	halo_data["time_percent"] += [calc_avg_percent]
	halo_data["halo_percent"] += [halo_time_percent]
	halo_data["halo_details"] += [halo_time_details]

	halo_data_weak = halo_times_weak[key_weak]
	halo_data_weak["VINIT"] += [vertices_init]
	halo_data_weak["V"] += [vertices]
	halo_data_weak["E"] += [elements]
	halo_data_weak["border"] += [max_border]
	halo_data_weak["proc"] += [proc]
	halo_data_weak["n"] += [n]
	halo_data_weak["p"] += [p]
	halo_data_weak["t"] += [t]
	halo_data_weak["time"] += [calc_avg_reduced]
	halo_data_weak["halo_time"] += [halo_time]
	halo_data_weak["time_percent"] += [calc_avg_percent]
	halo_data_weak["halo_percent"] += [halo_time_percent]
	halo_data_weak["halo_details"] += [halo_time_details]
[print(k,v) for k,v in sorted(halo_times.items())]
print()
# Plot comp and comm
if len(halo_times) > 0:
	key_max = fixed_key if fixed_key in halo_times else max(halo_times, key=lambda x: x[0])
	v_max = halo_times[key_max]
	order = np.argsort(v_max["n"])
	xs = [str(x) for x in np.array(v_max["n"])[order]]
	y_comm = np.array(v_max["halo_time"])[order]
	y_comp = np.array(v_max["time"])[order]
	plt.bar(xs, y_comm, label='Communication', bottom=y_comp)
	plt.bar(xs, y_comp, label='Computation')
	plt.legend()
	plt.xlabel('Nodes')
	plt.ylabel('Runtime [ms]')
	plt.ylim(bottom=0)
	plt.savefig(dir_plot + 'communication_overhead.png', bbox_inches='tight')
	plt.clf()
# Plot comp and comm weak scaling
if len(halo_times_weak) > 0:
	key_max = fixed_key if fixed_key in halo_times_weak else max(halo_times_weak, key=lambda x: x[0])
	v_max = halo_times_weak[key_max]
	order = np.argsort(v_max["n"])
	xs = [str(x) for x in np.array(v_max["n"])[order]]
	y_comm = np.array(v_max["halo_time"])[order]
	y_comp = np.array(v_max["time"])[order]
	plt.bar(xs, y_comm, label='Communication', bottom=y_comp)
	plt.bar(xs, y_comp, label='Computation')
	plt.legend(loc=4)
	plt.xlabel('Nodes')
	plt.ylabel('Runtime [ms]')
	plt.ylim(bottom=0)
	plt.savefig(dir_plot + 'communication_overhead_weak.png', bbox_inches='tight')
	plt.clf()
# Plot halo time
xticks_n = []
for k in sorted(halo_times, reverse=True):
	if k[0] != fixed_v:
		continue
	v = halo_times[k]
	if len(v["V"]) > n_halo_points:
		xticks_n += v["n"]
		order = np.argsort(v["n"])
		xs = np.array(v["n"])[order]
		ys = np.array(v["halo_time"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		#plt.plot(xs_cut, ys_cut, '-o', label='V total: {}'.format(k[0]))
		plt.plot(xs_cut, ys_cut, '-o', label='V total: {}, R:{}, T:{}'.format(k[0], max_proc//k[1], k[1]))
xticks_n = sorted(set(xticks_n))
plt.xticks(xticks_n)
plt.legend()
plt.xlabel('Nodes')
plt.ylabel('Communication time [ms]')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'communication_time.png', bbox_inches='tight')
plt.clf()
# Plot halo time weak
xticks_n = []
for k in sorted(halo_times_weak, reverse=True):
	if k[0] != fixed_v:
		continue
	v = halo_times_weak[k]
	if len(v["V"]) > n_halo_points:
		xticks_n += v["n"]
		order = np.argsort(v["n"])
		xs = np.array(v["n"])[order]
		ys = np.array(v["halo_time"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		#plt.plot(xs_cut, ys_cut, '-o', label='V per node: {}'.format(k[0]))
		plt.plot(xs_cut, ys_cut, '-o', label='V per node: {}, R:{}, T:{}'.format(k[0], max_proc//k[1], k[1]))
xticks_n = sorted(set(xticks_n))
plt.xticks(xticks_n)
plt.legend()
plt.xlabel('Nodes')
plt.ylabel('Communication time [ms]')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'communication_time_weak.png', bbox_inches='tight')
plt.clf()
# Plot communication between nodes and cores
xticks_n = []
halo_node_all = {}
halo_core_all = {}
for k in sorted(halo_times, reverse=True):
	if k[0] != fixed_v or k[1] not in [4,3]:
		continue
	v = halo_times[k]
	xticks_n += v["n"]
	order = np.argsort(v["n"])
	xs = np.array(v["n"])[order]
	ys = np.array(v["halo_details"], dtype=object)[order]
	halo_node = []
	halo_core = []
	xs_node = []
	xs_core = []
	for i in range(len(ys)):
		n = xs[i]
		halo_details_p = ys[i]
		halo_node_p = [halo_detail_p for halo_detail_p in halo_details_p if halo_detail_p > halo_core_limit]
		halo_core_p = [halo_detail_p for halo_detail_p in halo_details_p if halo_detail_p < halo_core_limit]
		if halo_node_p:
			halo_node_avg = np.mean(halo_node_p)
			halo_node += [halo_node_avg]
			xs_node += [xs[i]]
			if n not in halo_node_all:
				halo_node_all[n] = []
			halo_node_all[n] += [halo_node_avg]
		if halo_core_p:
			halo_core_avg = np.mean(halo_core_p)
			halo_core += [halo_core_avg]
			xs_core += [xs[i]]
			if n not in halo_core_all:
				halo_core_all[n] = []
			halo_core_all[n] += [halo_core_avg]
	#plt.plot(xs_node, halo_node, '-o', label='Node comm, R:{}, T:{}'.format(max_proc//k[1], k[1]))
	#plt.plot(xs_core, halo_core, '-o', label='Core comm, R:{}, T:{}'.format(max_proc//k[1], k[1]))
xticks_n = sorted(set(xticks_n))
halo_node_all_plot = []
halo_core_all_plot = []
for n in sorted(halo_node_all):
	halo_node_all_plot += [sum(halo_node_all[n]) / len(halo_node_all[n])]
for n in sorted(halo_core_all):
	halo_core_all_plot += [sum(halo_core_all[n]) / len(halo_core_all[n])]
if len(xticks_n) > 1:
	plt.plot(xticks_n[1:], halo_node_all_plot, '-o', label='Node communication')
	plt.plot(xticks_n, halo_core_all_plot, '-o', label='Core communication')
	plt.xticks(xticks_n)
	plt.legend(bbox_to_anchor=[1.0, 0.5])
	plt.xlabel('Nodes')
	plt.ylabel('Communication time [ms]')
	plt.ylim(bottom=0)
	plt.savefig(dir_plot + 'communication_node_core.png', bbox_inches='tight')
	plt.clf()
# Plot communication between nodes and cores over border size
xticks_b = []
borders = {}
halo_details_list = {}
halo_nodes = []
halo_nodes1 = []
halo_nodes2 = []
halo_cores = []
halo_nodes_print = []
halo_nodes1_print = []
halo_nodes2_print = []
for k in sorted(halo_times):
	if k[1] not in [4,3]:
		continue
	v = halo_times[k]
	border_avg = sum(v["border"])//len(v["border"])
	if k[0] in borders:
		borders[k[0]] = (borders[k[0]] + border_avg)//2
		halo_details_list[k[0]] += v["halo_details"]
	else:
		borders[k[0]] = border_avg
		halo_details_list[k[0]] = v["halo_details"]
borders = [borders[k] for k in sorted(borders)]
halo_details_list = [halo_details_list[k] for k in sorted(halo_details_list)]
xticks_b = borders[::]
for i in range(len(borders)):
	border = borders[i]
	halo_details = halo_details_list[i]
	halo_node = []
	halo_node1 = []
	halo_node2 = []
	halo_core = []
	print(border)
	for j in range(len(halo_details)):
		print([x for x in halo_details_p if x > halo_core_limit])
		halo_details_p = halo_details[j]
		halo_nodes_print += [halo_detail_p for halo_detail_p in halo_details_p if halo_detail_p > halo_core_limit and halo_detail_p < halo_node_limit0]
		halo_nodes1_print += [halo_detail_p for halo_detail_p in halo_details_p if halo_detail_p > halo_node_limit0 and halo_detail_p < halo_node_limit1]
		halo_nodes2_print += [halo_detail_p for halo_detail_p in halo_details_p if halo_detail_p > halo_node_limit1]
		halo_node += [halo_detail_p for halo_detail_p in halo_details_p if halo_detail_p > halo_core_limit and halo_detail_p < halo_node_limit0]
		halo_node1 += [halo_detail_p for halo_detail_p in halo_details_p if halo_detail_p > halo_node_limit0 and halo_detail_p < halo_node_limit1]
		halo_node2 += [halo_detail_p for halo_detail_p in halo_details_p if halo_detail_p > halo_node_limit1]
		halo_core += [halo_detail_p for halo_detail_p in halo_details_p if halo_detail_p < halo_core_limit]
	print(halo_node)
	print(halo_node1)
	print(halo_node2)
	print()
	if halo_node:
		halo_nodes += [sum(halo_node) / len(halo_node)]
	if halo_node1:
		halo_nodes1 += [sum(halo_node1) / len(halo_node1)]
	if halo_node2:
		halo_nodes2 += [sum(halo_node2) / len(halo_node2)]
	if halo_core:
		halo_cores += [sum(halo_core) / len(halo_core)]
xticks_b = sorted(set(xticks_b))
print(halo_nodes)
print(halo_nodes1)
print(halo_nodes2)
print()
print()
print(halo_nodes_print)
nodes_avg = sum(halo_nodes_print) / len(halo_nodes_print)
nodes_var = sum([(x - nodes_avg)**2 for x in halo_nodes_print]) / (len(halo_nodes_print) - 1)
print(nodes_avg, '+-', np.sqrt(nodes_var))
print(halo_nodes1_print)
nodes1_avg = sum(halo_nodes1_print) / len(halo_nodes1_print)
nodes1_var = sum([(x - nodes1_avg)**2 for x in halo_nodes1_print]) / (len(halo_nodes1_print) - 1)
print(nodes1_avg, '+-', np.sqrt(nodes1_var))
print(halo_nodes2_print)
nodes2_avg = sum(halo_nodes2_print) / len(halo_nodes2_print)
nodes2_var = sum([(x - nodes2_avg)**2 for x in halo_nodes2_print]) / (len(halo_nodes2_print) - 1)
print(nodes2_avg, '+-', np.sqrt(nodes2_var))
print()
plt.xticks(xticks_b)
plt.plot(xticks_b, halo_nodes, '-o', label='Node communication')
plt.plot(xticks_b, halo_cores, '-o', label='Core communication')
plt.plot(xticks_b[:2], halo_nodes1, '-o', color='tab:blue')
plt.plot(xticks_b[1], halo_nodes2, '-o', color='tab:blue')
plt.legend()
plt.xlabel('Halo triangles')
plt.ylabel('Communication time [ms]')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'communication_node_core_border.png', bbox_inches='tight')
plt.clf()


# Plot prediction
max_mem_s_K = -1
prediction_file = dir_input + "prediction_roof.txt"
if os.path.isfile(prediction_file):
	with open(prediction_file, 'r') as f:
		lines = [line.split() for line in f.readlines()]
		max_mem_s_K = float(lines[0][0]) * 1e6
strong_prediction = {}
weak_prediction = {}
for raw_file in raw_files:
	data = raw_data[raw_file]
	structure_file = data[0]
	n = data[1]
	p = data[2]
	t = data[3]
	vertices_init = data[4]
	vertices = data[5]
	elements = data[6]
	vertices_per_n = vertices_init // n
	mem_tot = structure_data[structure_file][8]

	if t not in labels_limit_t:
		continue
	key_strong = (vertices_init, t)
	key_weak = (vertices_per_n, t)

	if max_mem_s_K == -1:
		time_prediction = mem_tot / 1e9 / t
	else:
		time_prediction = mem_tot / max_mem_s_K / t

	if key_strong not in strong_prediction:
		strong_prediction[key_strong] = {"VINIT":[], "V":[], "E":[], "n":[], "p":[], "t":[], "time_prediction":[]}

	if key_weak not in weak_prediction:
		weak_prediction[key_weak] = {"VINIT":[], "V":[], "E":[], "n":[], "p":[], "t":[], "time_prediction":[]}

	strong_data = strong_prediction[key_strong]
	strong_data["VINIT"] += [vertices_init]
	strong_data["V"] += [vertices]
	strong_data["E"] += [elements]
	strong_data["n"] += [n]
	strong_data["p"] += [p]
	strong_data["t"] += [t]
	strong_data["time_prediction"] += [time_prediction]

	weak_data = weak_prediction[key_weak]
	weak_data["VINIT"] += [vertices_init]
	weak_data["V"] += [vertices]
	weak_data["E"] += [elements]
	weak_data["n"] += [n]
	weak_data["p"] += [p]
	weak_data["t"] += [t]
	weak_data["time_prediction"] += [time_prediction]
# Plot strong prediction
strong_xticks_n = []
for k in sorted(strong_prediction, reverse=True):
	v = strong_prediction[k]
	if len(set(v["n"])) > n_points_plot and 1 in v["n"]:
		strong_xticks_n += v["n"]
		order = np.argsort(v["n"])
		xs = np.array(v["n"])[order]
		ys = np.array(v["time_prediction"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		plt.plot(xs_cut, ys_cut, '-o', label='V total: {}, R:{}, T:{}'.format(k[0], max_proc//k[1], k[1]))
strong_xticks_n = sorted(set(strong_xticks_n))
plt.xticks(strong_xticks_n)
if max_mem_s_K == -1:
	ylabel = 'Memory operations [GB]'
else:
	ylabel = 'Predicted runtime [ms]'
plt.legend()
plt.xlabel(xlabel)
plt.ylabel(ylabel)
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'time_strong_predicion.png', bbox_inches='tight')
plt.clf()
# Plot weak prediction
weak_xticks_n = []
for k in sorted(weak_prediction, reverse=True):
	v = weak_prediction[k]
	if len(set(v["n"])) > n_points_plot and 1 in v["n"]:
		weak_xticks_n += v["n"]
		order = np.argsort(v["n"])
		xs = np.array(v["n"])[order]
		ys = np.array(v["time_prediction"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		plt.plot(xs_cut, ys_cut, '-o', label='V per node: {}, R:{}, T:{}'.format(k[0], max_proc//k[1], k[1]))
weak_xticks_n = sorted(set(weak_xticks_n))
plt.xticks(weak_xticks_n)
if max_mem_s_K == -1:
	ylabel = 'Memory operations [GB]'
else:
	ylabel = 'Predicted runtime [ms]'
plt.legend()
plt.xlabel(xlabel)
plt.ylabel(ylabel)
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'time_weak_prediction.png', bbox_inches='tight')
plt.clf()


# Plot time, speedup and efficiency graf
strong_scale = {}
strong_serial = {}
weak_scale = {}
weak_serial = {}
for raw_file in raw_files:
	data = raw_data[raw_file]
	n = data[1]
	p = data[2]
	t = data[3]
	vertices_init = data[4]
	vertices = data[5]
	elements = data[6]
	calc_avg_reduced = data[8]
	vertices_per_n = vertices_init // n

	if t not in labels_limit_t:
		continue
	key_strong = (vertices_init, t)
	key_weak = (vertices_per_n, t)

	if key_strong not in strong_scale:
		strong_scale[key_strong] = {"VINIT":[], "V":[], "E":[], "n":[], "p":[], "t":[], "time":[], "speedup":[], "efficiency":[]}

	if key_weak not in weak_scale:
		weak_scale[key_weak] = {"VINIT":[], "V":[], "E":[], "n":[], "p":[], "t":[], "time":[], "speedup":[], "efficiency":[]}

	if n == 1:
		strong_serial[key_strong] = {"n":n, "time": calc_avg_reduced}
		weak_serial[key_weak] = {"n":n, "time": calc_avg_reduced}

	strong_data = strong_scale[key_strong]
	strong_data["VINIT"] += [vertices_init]
	strong_data["V"] += [vertices]
	strong_data["E"] += [elements]
	strong_data["n"] += [n]
	strong_data["p"] += [p]
	strong_data["t"] += [t]
	strong_data["time"] += [calc_avg_reduced]

	weak_data = weak_scale[key_weak]
	weak_data["VINIT"] += [vertices_init]
	weak_data["V"] += [vertices]
	weak_data["E"] += [elements]
	weak_data["n"] += [n]
	weak_data["p"] += [p]
	weak_data["t"] += [t]
	weak_data["time"] += [calc_avg_reduced]
for k, v in strong_scale.items():
	if k in strong_serial:
		time_serial = strong_serial[k]["time"]
		for i in range(len(v["V"])):
			speedup = time_serial / v["time"][i]
			efficiency = speedup / v["n"][i]
			v["speedup"] += [speedup]
			v["efficiency"] += [efficiency]
for k, v in weak_scale.items():
	if k in weak_serial:
		time_serial = weak_serial[k]["time"]
		for i in range(len(v["V"])):
			speedup = v["n"][i] * time_serial / v["time"][i]
			efficiency = speedup / v["n"][i]
			v["speedup"] += [speedup]
			v["efficiency"] += [efficiency]
# Plot time per node
strong_xticks_n = []
for k in sorted(strong_serial, reverse=True):
	v = strong_scale[k]
	if len(v["V"]) > n_points_plot:
		strong_xticks_n += v["n"]
		order = np.argsort(v["n"])
		xs = np.array(v["n"])[order]
		ys = np.array(v["time"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		plt.plot(xs_cut, ys_cut, '-o', label='V total: {}, R:{}, T:{}'.format(k[0], max_proc//k[1], k[1]))
strong_xticks_n = sorted(set(strong_xticks_n))
plt.xticks(strong_xticks_n)
plt.legend()
plt.xlabel(xlabel)
plt.ylabel('Runtime [ms]')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'time_strong.png', bbox_inches='tight')
plt.clf()
weak_xticks_n = []
for k in sorted(weak_serial, reverse=True):
	v = weak_scale[k]
	if len(v["V"]) > n_points_plot:
		weak_xticks_n += v["n"]
		order = np.argsort(v["n"])
		xs = np.array(v["n"])[order]
		ys = np.array(v["time"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		plt.plot(xs_cut, ys_cut, '-o', label='V per node: {}, R:{}, T:{}'.format(k[0], max_proc//k[1], k[1]))
weak_xticks_n = sorted(set(weak_xticks_n))
plt.xticks(weak_xticks_n)
plt.legend()
plt.xlabel(xlabel)
plt.ylabel('Runtime [ms]')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'time_weak.png', bbox_inches='tight')
plt.clf()
# Plot speedup
for k in sorted(strong_serial, reverse=True):
	v = strong_scale[k]
	if len(v["V"]) > n_points_plot:
		order = np.argsort(v["n"])
		xs = np.array(v["n"])[order]
		ys = np.array(v["speedup"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		plt.plot(xs_cut, ys_cut, '-o', label='V total: {}, R:{}, T:{}'.format(k[0], max_proc//k[1], k[1]))
plt.xticks(strong_xticks_n)
plt.legend()
plt.xlabel(xlabel)
plt.ylabel('Speedup')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'speedup_strong.png', bbox_inches='tight')
plt.clf()
for k in sorted(weak_serial, reverse=True):
	v = weak_scale[k]
	if len(v["V"]) > n_points_plot:
		order = np.argsort(v["n"])
		xs = np.array(v["n"])[order]
		ys = np.array(v["speedup"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		plt.plot(xs_cut, ys_cut, '-o', label='V per node: {}, R:{}, T:{}'.format(k[0], max_proc//k[1], k[1]))
plt.xticks(weak_xticks_n)
plt.legend()
plt.xlabel(xlabel)
plt.ylabel('Speedup')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'speedup_weak.png', bbox_inches='tight')
plt.clf()
# Plot efficiency
for k in sorted(strong_serial, reverse=True):
	v = strong_scale[k]
	if len(v["V"]) > n_points_plot:
		order = np.argsort(v["n"])
		xs = np.array(v["n"])[order]
		ys = np.array(v["efficiency"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		plt.plot(xs_cut, ys_cut, '-o', label='V total: {}, R:{}, T:{}'.format(k[0], max_proc//k[1], k[1]))
plt.xticks(strong_xticks_n)
plt.legend()
plt.xlabel(xlabel)
plt.ylabel('Efficiency')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'efficiency_strong.png', bbox_inches='tight')
plt.clf()
for k in sorted(weak_serial, reverse=True):
	v = weak_scale[k]
	if len(v["V"]) > n_points_plot:
		order = np.argsort(v["n"])
		xs = np.array(v["n"])[order]
		ys = np.array(v["efficiency"])[order]
		xs_cut = sorted(set(xs))
		ys_cut = [ys[0]]
		x_last = xs[0]
		yi=0
		for i in range(1, len(xs)):
			if x_last == xs[i]:
				if ys[i] > ys_cut[yi]:
					ys_cut[yi] = ys[i]
			else:
				ys_cut += [ys[i]]
				yi += 1
			x_last = xs[i]
		plt.plot(xs_cut, ys_cut, '-o', label='V per node: {}, R:{}, T:{}'.format(k[0], max_proc//k[1], k[1]))
plt.xticks(weak_xticks_n)
plt.legend()
plt.xlabel(xlabel)
plt.ylabel('Efficiency')
plt.ylim(bottom=0)
plt.savefig(dir_plot + 'efficiency_weak.png', bbox_inches='tight')
plt.clf()
[print(k,strong_scale[k]) for k in strong_scale]
[print('-',k,strong_serial[k]) for k in strong_serial]
print()
[print(k,weak_scale[k]) for k in weak_scale]
[print('-',k,weak_serial[k]) for k in weak_serial]
