from scipy.spatial import Delaunay
from sys import argv
from math import sqrt
from PIL import Image
import numpy as np
import matplotlib.pyplot as plt

# Check if we got a specified number of points, default otherwise
if ( len(argv) > 1 ):
    n_points = int(argv[1])
else:
    n_points = 128

points = []
colormap = {}
color_index = {}
width = 2088 #specific for this image
height = 1638 #specific for this image
x_max = 63.5 #specific for this image
y_max = 49.8 #specific for this image
side_avg = (x_max+y_max)/2
# water coverage of enitre image
#water_coverage = [0.17923602, 0.48261974] #specific for this image
water_coverage = [0.11549513704686118, 0.06362773029439696, 0.23201683905706894, 0.2504602145406743] #specific for this image
water_coverage_sum = sum(water_coverage)
# colors [green, blue]
#colors = [(0,255,0), (0,0,255)] #specific for this image
colors = [(0,255,0,255), (255,0,0,255), (0,0,255,255), (255,255,0,255)] #specific for this image
n_colors = len(colors)
color_index = {colors[i]:i for i in range(n_colors)}
# set distribution of points for each color
#color_density = [0.9, 0.1]
#color_density = [50,25,9,2]
color_density = [65, 25, 8, 2]
# number of points divided by constant k
# p_i / (w_coverage_i/w_coverage_sum) = color_density_i * k
# p_i / k = (w_coverage_i/w_coverage_sum) * color_density_i
points_div_k = [(water_coverage[i]/water_coverage_sum) * color_density[i] for i in range(n_colors)]
# find k
# since sum(points_partition) = n_points
# => n_points / k = sum(points_partition / k)
# => n_points / k = sum(points_div_k)
k = n_points / sum(points_div_k)
# calculate points for each color
points_partition = [p_div_k * k for p_div_k in points_div_k]
# add points propotional to the coverage to cover entire map
# since the land area is checked and removed afterwards
n_points_side = [int(sqrt(points_partition[i]/water_coverage[i])) for i in range(n_colors)]
n_points_x = [int(p * (1+(x_max - side_avg)/side_avg)) for p in n_points_side]
n_points_y = [int(p * (1+(y_max - side_avg)/side_avg)) for p in n_points_side]
xh = [x_max / p for p in n_points_x]
yh = [y_max / p for p in n_points_y]
xh_pix = [width / p for p in n_points_x]
yh_pix = [height / p for p in n_points_y]
upper_len = [sqrt(xh[i]**2 + yh[i]**2) * 1.00001 for i in range(n_colors)]
max_len = max(upper_len)

#im = Image.open("plot/mehamn.png")
im = Image.open("plot/mehamn2.png")
pixels = im.load()

# Code for finding relevant data for the specific image
#print(im.size)
#test_area = width*height
#test_colors = []
#test_color_index = {}
#test_color_coverage = []
#test_n_colors = 0
#for x in range(width):
#	for y in range(height):
#		color = pixels[x,y]
#		if color in test_colors:
#			test_color_coverage[test_color_index[color]] += 1
#		elif color != (0,0,0,255) and color != (0,0,0):
#			test_colors.append(color)
#			test_color_index[color] = test_n_colors
#			test_color_coverage.append(1)
#			test_n_colors += 1
#print(test_area, sum(test_color_coverage)/test_area)
#for i in range(test_n_colors):
#	print(test_colors[i], test_color_coverage[i], test_color_coverage[i]/test_area)

#tmp_x = [[], [], [], []]
#tmp_y = [[], [], [], []]
for i_color in range(n_colors):
	for x in range(n_points_x[i_color]):
		for y in range(n_points_y[i_color]):
			coord_x = int(x*xh_pix[i_color])
			coord_y = int(y*yh_pix[i_color])
			color = pixels[coord_x,coord_y]
			#tmp_x[i_color].append(x*xh[i_color])
			#tmp_y[i_color].append(y*yh[i_color])
			if (color == colors[i_color]):
				point = (x*xh[i_color],y*yh[i_color])
				points.append(point)
				colormap[point] = color
#tmp_x = [np.array(tmp_x[i]) for i in range(4)]
#tmp_y = [np.array(tmp_y[i]) for i in range(4)]
"""
#plot four subplots
fig,axs = plt.subplots(2,2)
axs[0,0].plot(tmp_x[0], tmp_y[0], 'o', color='g')
axs[0,0].set_title('Green area', fontsize=30)
axs[0,0].invert_yaxis()
axs[0,1].plot(tmp_x[1], tmp_y[1], 'o', color='r')
axs[0,1].set_title('Red area', fontsize=30)
axs[0,1].invert_yaxis()
axs[1,0].plot(tmp_x[2], tmp_y[2], 'o', color='b')
axs[1,0].set_title('Blue area', fontsize=30)
axs[1,0].invert_yaxis()
axs[1,1].plot(tmp_x[3], tmp_y[3], 'o', color='y')
axs[1,1].set_title('Yellow area', fontsize=30)
axs[1,1].invert_yaxis()
plt.setp(axs, xlim=[0,x_max], ylim=[0,y_max])
"""
"""
#plot one map
plt.plot(tmp_x[0],tmp_y[0], 'o', color='g')
plt.plot(tmp_x[1],tmp_y[1], 'o', color='r')
plt.plot(tmp_x[2],tmp_y[2], 'o', color='b')
plt.plot(tmp_x[3],tmp_y[3], 'o', color='y')
plt.gca().invert_yaxis()
"""
#plt.show()

# Triangulate the points
tri = Delaunay ( points )

# Remove triangles crossing land
new_simplices = []
for element in tri.simplices:
	is_valid = True
	for p in range(3):
		p1 = points[element[p]]
		p2 = points[element[(p+1)%3]]
		length = sqrt((p2[0]-p1[0])**2 + (p2[1]-p1[1])**2)
		color1 = colormap[p1]
		color2 = colormap[p2]
		color_index1 = color_index[color1]
		color_index2 = color_index[color2]
		max_len1 = upper_len[color_index1]
		max_len2 = upper_len[color_index2]
		middle_ratio = max_len1 / (max_len1 + max_len2)
		x_middle = p1[0] + (p2[0] - p1[0]) * middle_ratio
		y_middle = p1[1] + (p2[1] - p1[1]) * middle_ratio
		p_middle = [x_middle, y_middle]
		x_pix = int(p_middle[0]/x_max * width)
		y_pix = int(p_middle[1]/y_max * height)
		color_middle = pixels[x_pix, y_pix]
		if color1 == color2:
			if length > max_len1*1.5 or color_middle == (0,0,0,255) or color_middle == (0,0,0):
				is_valid = False
				break
		else:
			if length > max(max_len1,max_len2)*2 or color_middle == (0,0,0,255) or color_middle == (0,0,0):
				is_valid = False
				break
	if is_valid:
		new_simplices.append(element)
new_simplices = np.array(new_simplices)

# Plot triangles
#print(len(points))
#plt.triplot(tri.points[:,0], tri.points[:,1], new_simplices)
#plt.plot(tri.points[:,0], tri.points[:,1], 'o')
#plt.gca().invert_yaxis()
#plt.show()

# Print mesh in SALOME dat format
print ( len(points), len(new_simplices), 0.0, 0.0, x_max, y_max )
for i in range(len(points)):
	print ( i+1, points[i][0], points[i][1], 0.0 )
for i in range(len(new_simplices)):
	print ( i+1, 203, new_simplices[i][0]+1, new_simplices[i][1]+1, new_simplices[i][2]+1 )
