import sys

def reformat(inputfile, outdir, n_rank):
	# Declare variables
	n_vertices = n_elements = min_x = min_y = max_x = max_y = 0
	n_neumann_edges = 0
	vertices = {}
	elements = {}
	vertices_sorted = []
	elements_sorted = []
	elements_sorted_id = []
	edges = {}

	# Read vertices and elements from file
	# Elements ids and vertices ids cannot be bigger than the amount of elements and vertices
	with open(inputfile) as f:
		(n_vertices, n_elements, min_x, min_y, max_x, max_y) = f.readline().split()
		n_vertices, n_elements = int(n_vertices), int(n_elements)
		min_x, min_y, max_x, max_y = float(min_x), float(min_y), float(max_x), float(max_y)

		vertices = [list(map(float, f.readline().split())) for _ in range(n_vertices)]
		elements = [list(map(int, f.readline().split())) for _ in range(n_elements)]
	
	# Create dictionaries for vertices and elements with id as key
	vertices = {int(line[0]): line[1:] for line in vertices}
	elements = {line[0]: [sorted(line[2:]), 0, []] for line in elements}
	
	# Find neighbor elements
	for idx, values in elements.items():
		v = values[0]
		edge1 = (v[0], v[1])
		edge2 = (v[0], v[2])
		edge3 = (v[1], v[2])
		if (edge1 in edges):
			neighbor_idx = edges[edge1][0]
			edges[edge1][1] = idx
			values[1] += 1
			values[2] += [neighbor_idx]
			elements[neighbor_idx][1] += 1
			elements[neighbor_idx][2] += [idx]
		else:
			edges[edge1] = [idx, -1]
		if (edge2 in edges):
			neighbor_idx = edges[edge2][0]
			edges[edge2][1] = idx
			values[1] += 1
			values[2] += [neighbor_idx]
			elements[neighbor_idx][1] += 1
			elements[neighbor_idx][2] += [idx]
		else:
			edges[edge2] = [idx, -1]
		if (edge3 in edges):
			neighbor_idx = edges[edge3][0]
			edges[edge3][1] = idx
			values[1] += 1
			values[2] += [neighbor_idx]
			elements[neighbor_idx][1] += 1
			elements[neighbor_idx][2] += [idx]
		else:
			edges[edge3] = [idx, -1]
	# Set empty neighbors to -1
	for idx, values in elements.items():
		n_size = values[1]
		n = values[2]
		for _ in range(n_size, 3):
			n += [-1]

	# Create lists of vertices and elements sorted by y axis
	vertices_sorted = [(k, v) for k, v in vertices.items()]
	vertices_sorted = sorted(vertices_sorted, key=lambda line: (line[1][1], line[1][0]))
	elements_sorted = [[k, v[0], \
					   [(vertices[v[0][0]][0] + vertices[v[0][1]][0] + vertices[v[0][2]][0]) / 3, \
						(vertices[v[0][0]][1] + vertices[v[0][1]][1] + vertices[v[0][2]][1]) / 3, \
						(vertices[v[0][0]][2] + vertices[v[0][1]][2] + vertices[v[0][2]][2]) / 3], \
					   v[1], \
					   v[2]] \
					  for k, v in elements.items()]
	elements_sorted = sorted(elements_sorted, key=lambda line: (line[2][1], line[2][0]))
	elements = {line[0]: line[1:] for line in elements_sorted}
	elements_sorted_id = [line[0] for line in elements_sorted]

	# Find local sizes
	n_vertices_local = 0
	if (n_elements % n_rank) == 0:
		n_elements_local = n_elements // n_rank
		n_elements_local_max = n_elements_local
	else:
		n_elements_local = (n_elements // n_rank) + 1
		n_elements_local_max = n_elements_local

	def area(v):
		v1 = vertices[v[0]]
		v2 = vertices[v[1]]
		v3 = vertices[v[2]]
		x1,x2,x3 = v1[0], v2[0], v3[0]
		y1,y2,y3 = v1[1], v2[1], v3[1]
		return abs(0.5 * (x1*(y2-y3) + x2*(y3-y1) + x3*(y1-y2)))
	

	# Find local values and write to output file for each rank
	for rank in range(n_rank):
		if rank == (n_rank - 1) and (n_elements % n_rank) != 0:
			n_elements_local = n_elements % n_elements_local_max

		# Local values
		n_neumann_edges_local = 0
		vertices_local = {}
		vertices_sorted_local = []
		elements_local = {}
		elements_sorted_local = []
		elements_upper_border = [] # Just within local elements
		elements_lower_border = [] # Just within local elements
		elements_upper_halo = [] # Outside the local elements
		elements_lower_halo = [] # Outside the local elements
		elements_neumann_edges = [] # Elements mirrored from edges

		start_index = rank * n_elements_local_max
		elements_sorted_local = [elements_sorted[i] for i in range(start_index, start_index + n_elements_local)]
		elements_local = {line[0]: elements[line[0]] for line in elements_sorted_local}

		# Create mirror elements for Neumann boundary conditions
		for line in elements_sorted_local:
			idx = line[0]
			v = line[1]
			c = line[2]
			edge1 = (v[0], v[1])
			edge2 = (v[0], v[2])
			edge3 = (v[1], v[2])
			edges_tmp = [(edge1, v[2]), (edge2, v[1]), (edge3, v[0])]
			n_neumann_edges_local += 3 - line[3]
			for edge_it in edges_tmp:
				edge = edge_it[0]
				pair = edges[edge]
				nidx = pair[0] if idx != pair[0] else pair[1]
				if nidx == -1:
					n_neumann_edges += 1
					mirror_e_id = n_elements + n_neumann_edges
					mirror_v_id = n_vertices + n_neumann_edges
					mirror_v = [edge[0], edge[1], mirror_v_id]

					# Using mirror formula from https://math.stackexchange.com/questions/1013230/how-to-find-coordinates-of-reflected-point
					x_org = vertices[edge_it[1]][0]
					y_org = vertices[edge_it[1]][1]
					if (vertices[edge[0]][0] - vertices[edge[1]][0] == 0):
						x_mirror = vertices[edge[0]][0] + (vertices[edge[0]][0] - x_org);
						y_mirror = y_org
					else:
						a = (vertices[edge[0]][1] - vertices[edge[1]][1]) / (vertices[edge[0]][0] - vertices[edge[1]][0])
						b = -1.0
						c = -a*vertices[edge[0]][0] - b*vertices[edge[0]][1]
						k = -2 * (a*x_org + b*y_org + c) / (a**2 + b**2)
						x_mirror = k*a + x_org
						y_mirror = k*b + y_org
					vertices[mirror_v_id] = [x_mirror, y_mirror, 0.0]

					mirror_c = [(vertices[mirror_v[0]][0] + vertices[mirror_v[1]][0] + vertices[mirror_v[2]][0]) / 3,\
								(vertices[mirror_v[0]][1] + vertices[mirror_v[1]][1] + vertices[mirror_v[2]][1]) / 3,\
								(vertices[mirror_v[0]][2] + vertices[mirror_v[1]][2] + vertices[mirror_v[2]][2]) / 3]
					mirror_n_neighbors = 1
					mirror_neighbors = [idx, -1, -1]
					elements_neumann_edges += [[mirror_e_id, mirror_v, mirror_c, mirror_n_neighbors, mirror_neighbors]]
					for i in range(3):
						if line[4][i] == -1:
							line[4][i] = mirror_e_id
							break

		# Find borders
		for line in elements_sorted_local:
			idx = line[0]
			v = line[1]
			edge1 = (v[0], v[1])
			edge2 = (v[0], v[2])
			edge3 = (v[1], v[2])
			edges_tmp = [edge1, edge2, edge3]
			for edge in edges_tmp:
				pair = edges[edge]
				nidx = pair[0] if idx != pair[0] else pair[1]
				if nidx != -1 and nidx not in elements_local:
					nidx_index = elements_sorted_id.index(nidx)
					if nidx_index < start_index:
						elements_upper_border += [idx]
						elements_upper_halo += [nidx]
					else:
						elements_lower_border += [idx]
						elements_lower_halo += [nidx]
		elements_upper_border = sorted(list(set(elements_upper_border)))
		elements_lower_border = sorted(list(set(elements_lower_border)))
		elements_upper_halo = sorted(list(set(elements_upper_halo)))
		elements_lower_halo = sorted(list(set(elements_lower_halo)))
		# Temporary remove border elements
		elements_sorted_local = [line for line in elements_sorted_local if line[0] not in elements_upper_border and line[0] not in elements_lower_border]
		# Get the rest of the values for border elements
		elements_upper_border = [[idx] + elements[idx] for idx in elements_upper_border]
		elements_lower_border = [[idx] + elements[idx] for idx in elements_lower_border]
		elements_upper_halo = [[idx] + elements[idx] for idx in elements_upper_halo]
		elements_lower_halo = [[idx] + elements[idx] for idx in elements_lower_halo]
		# Move all border elements to the end of the list
		elements_sorted_local += elements_upper_border + elements_lower_border + elements_upper_halo + elements_lower_halo
		n_elements_local_with_halo = n_elements_local + len(elements_upper_halo) + len(elements_lower_halo)

		# Add neumann elements
		elements_sorted_local += elements_neumann_edges

		# Add all vertices connected to local elements
		for line in elements_sorted_local:
			v = line[1]
			vertices_local[v[0]] = vertices[v[0]]
			vertices_local[v[1]] = vertices[v[1]]
			vertices_local[v[2]] = vertices[v[2]]
		for line in elements_upper_halo:
			v = line[1]
			vertices_local[v[0]] = vertices[v[0]]
			vertices_local[v[1]] = vertices[v[1]]
			vertices_local[v[2]] = vertices[v[2]]
		for line in elements_lower_halo:
			v = line[1]
			vertices_local[v[0]] = vertices[v[0]]
			vertices_local[v[1]] = vertices[v[1]]
			vertices_local[v[2]] = vertices[v[2]]
		for line in elements_neumann_edges:
			v = line[1]
			vertices_local[v[2]] = vertices[v[2]]
		vertices_sorted_local = [(k, v) for k, v in vertices_local.items()]
		vertices_sorted_local = sorted(vertices_sorted_local, key=lambda line: (line[1][1], line[1][0]))
		n_vertices_local = len(vertices_sorted_local)

		# Write to output file
		with open(outdir + "/" + str(rank) + ".txt", 'w') as f:
			f.write("{} {} {:.6f} {:.6f} {:.6f} {:.6f}\n".format(n_vertices, n_elements, min_x, min_y, max_x, max_y))
			f.write("{} {} {} {}\n".format(n_vertices_local, n_elements_local, n_elements_local_max, n_neumann_edges_local))
			f.write("{} {} {} {}\n".format(len(elements_upper_border), len(elements_lower_border), len(elements_upper_halo), len(elements_lower_halo)))
			for line in elements_sorted_local:
				idx = line[0]
				v = line[1]
				c = line[2]
				n_size = line[3]
				n = line[4]
				f.write("{} {} {} {} {:.6f} {:.6f} {:.6f} {:.6f} {} {} {} {}\n".format(idx, v[0], v[1], v[2], c[0], c[1], c[2], area(v), n_size, n[0], n[1], n[2]))
			[f.write("{} {:.6f} {:.6f} {:.6f}\n".format(line[0], line[1][0], line[1][1], line[1][2])) for line in vertices_sorted_local]

if __name__ == "__main__":
	inputfile = sys.argv[1]
	outdir = sys.argv[2]
	n_rank = int(sys.argv[3])
	reformat(inputfile, outdir, n_rank)

