#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

typedef double real_t;
typedef int64_t int_t;

typedef struct {
    real_t x[3];
} vertex_t;

typedef struct {
    int_t v[3];
} triangle_t;

vertex_t *vertices = NULL;
triangle_t *faces = NULL;

int
main ()
{
    int_t n_vertices, n_elements;
    FILE *in = stdin;
    int err = fscanf ( in, "%ld %ld %*f %*f %*f %*f\n", &n_vertices, &n_elements );
    if ( err != 2 )
    {
        fprintf ( stderr, "Error: unable to read from file\n" );
        exit ( EXIT_FAILURE );
    }
    vertices = malloc ( n_vertices * sizeof(vertex_t) );
    faces = malloc ( n_elements * sizeof(triangle_t) );
    for ( int_t idx=0; idx<n_vertices; idx++ )
    {
        int_t n;
        real_t z,y,x;
        err = fscanf ( in, "%ld %lf %lf %lf\n", &n, &x, &y, &z );
        if ( err != 4 )
        {
            fprintf ( stderr, "Error: unable to read from file\n" );
            exit ( EXIT_FAILURE );
        }
        vertices[n-1].x[0] = x;
        vertices[n-1].x[1] = y;
        vertices[n-1].x[2] = z;
    }

    for ( int_t idx=0; idx<n_elements; idx++ )
    {
        int_t n, type, v0, v1, v2;
        err = fscanf ( in, "%ld %ld %ld %ld %ld\n",
            &n, &type, &v0, &v1, &v2
        );
        if ( err != 5 )
        {
            fprintf ( stderr, "Error: unable to read from file\n" );
            exit ( EXIT_FAILURE );
        }
        if ( type != 203 )
        {
            fprintf ( stderr, "Error: non-triangular element in mesh\n" );
            exit ( EXIT_FAILURE );
        }
        faces[n-1].v[0] = v0-1;
        faces[n-1].v[1] = v1-1;
        faces[n-1].v[2] = v2-1;
    }
    fclose ( in );

    for ( int_t idx=0; idx<n_elements; idx++ )
    {
        printf ( "%lf %lf %lf\n",
            vertices[faces[idx].v[0]].x[0], 
            vertices[faces[idx].v[0]].x[1], 
            vertices[faces[idx].v[0]].x[2] 
        );
        printf ( "%lf %lf %lf\n",
            vertices[faces[idx].v[1]].x[0], 
            vertices[faces[idx].v[1]].x[1], 
            vertices[faces[idx].v[1]].x[2] 
        );
        printf ( "%lf %lf %lf\n",
            vertices[faces[idx].v[2]].x[0], 
            vertices[faces[idx].v[2]].x[1], 
            vertices[faces[idx].v[2]].x[2] 
        );
        printf ( "%lf %lf %lf\n\n\n",
            vertices[faces[idx].v[0]].x[0], 
            vertices[faces[idx].v[0]].x[1], 
            vertices[faces[idx].v[0]].x[2] 
        );
    }

    free (vertices);
    free (faces);
}
