#define _XOPEN_SOURCE 500
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>

typedef int64_t int_t;
typedef double real_t;


void options ( int argc, char **argv );


int_t N = 128;
real_t h = 5e-2;


int
main ( int argc, char **argv )
{
    options ( argc, argv );

    int_t
        n_vertices = N*N,
        n_elements = 2*(N-1)*(N-1);
	
	real_t
		min_x = 0.0,
		min_y = 0.0,
		max_x = (N-1)*h,
		max_y = (N-1)*h;

    int_t idx;

    // Generate the header
    printf ( "%ld %ld %lf %lf %lf %lf\n", n_vertices, n_elements, min_x, min_y, max_x, max_y );

    // Generate the vertices
    idx = 0;
    for ( int_t y=0; y<N; y++ )
        for ( int_t x=0; x<N; x++ )
            printf ( "%ld %lf %lf %lf\n", ++idx, x*h, y*h, 0.0 );

    // Generate the elements
    idx = 0;
    for ( int_t y=0; y<N-1; y++ )
        for ( int_t x=0; x<N-1; x++ )
        {
            // Find the corners
            int_t
                north_west = ((y)   *N + (x)  )+1,
                north_east = ((y)   *N + (x+1))+1,
                south_west = ((y+1) *N + (x)  )+1,
                south_east = ((y+1) *N + (x+1))+1;

            // Print triangles
            if ( (x%2==0 && y%2==0) || (x%2==1 && y%2==1))
            {
                printf ( "%ld 203 %ld %ld %ld\n", ++idx,
                    north_west, north_east, south_west
                );
                printf ( "%ld 203 %ld %ld %ld\n", ++idx,
                    north_east, south_east, south_west
                );
            }
            else
            {
                printf ( "%ld 203 %ld %ld %ld\n", ++idx,
                    north_west, south_east, south_west
                );
                printf ( "%ld 203 %ld %ld %ld\n", ++idx,
                    north_east, south_east, north_west
                );
            }
        }

    exit ( EXIT_SUCCESS );
}


void
options ( int argc, char **argv )
{
    int o;
    while ( (o = getopt (argc, argv, "h:N:")) != -1 )
    {
        switch ( o )
        {
            case 'h': h = strtod ( optarg, NULL ); break;
            case 'N': N = strtol ( optarg, NULL, 10 ); break;
        }
    }
}
